!---------------------------------------------------------------------------------------
! FLEXINVERT : backgroundini
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates the background mixing ratio for each trajectory at each receptor by
!! multiplying the sensitivities to the initial mixing ratios with the mixing ratios from
!! another model (e.g. LMDZ4)
!!
!! Calls the following subroutines:
!!  - readinit: reads the gridded FLEXPART output files grid_initial_xxx containing the 
!!    sensitivity to the inital mixing ratios for a given receptor point
!!  - initial: reads mixing ratios from a global model and converts these to the flexpart
!!    grid for use with the initial sensitivities
!!    (note: to change to a custom initial mixing ratio file modify the subroutine 
!!     e.g. readlmdz.f90 in initial.f90)
!---------------------------------------------------------------------------------------

subroutine backgroundini(allobs,bkgnd_obs,bkgnd)

  use mod_var
  use mod_dates

  implicit none

  character(len=100) :: dirflexrec,filename,datestr
  integer :: numpoint,nobs,eomday
  integer :: yyyymmddhh,yyyymm,yyyy,mm,dd,hh,nr,nt,it,ix,jy,kz,ind,indm
  real(kind=8) :: jd,jtime
  real, dimension(nzmax) :: outheight
  real, dimension(numday*24/relperiod,numrec,ncrs) :: bkgnd
  real, dimension(numxgrid*numygrid) :: bkgndvec
  real, dimension(nobsteps,numrec,ncrs) :: bkgnd_obs
  real, dimension(:,:,:,:), allocatable :: init
  real, dimension(:), allocatable :: yini
  real, dimension(:,:,:,:), allocatable :: immr
  real, dimension(:), allocatable :: immrtime
  real(kind=8), dimension(nobsteps,numrec,6) :: allobs
  integer, parameter :: ntmax=1000  ! max no. timesteps in initial mixing ratio file

  fileinit='grid_initial_001'

  write(logid,*) 'Calculating background from initial conditions...'

  !--------------------------------------------------------------
  ! loop over receptors/stations
  !--------------------------------------------------------------

  do nr=1,numrec
    jd=juldatei
    ind=1
    indm=0

  !--------------------------------------------------------------
  ! loop over months in inversion
  !--------------------------------------------------------------

    do while(jd.lt.juldatef)
      yyyymmddhh=caldate(jd)
      yyyymm=yyyymmddhh/10000
      write(datestr,'(i6)') yyyymm
      eomday=calceomday(yyyymm)
      if(usedirflex) then
        dirflexrec=trim(dirflex)//trim(recname(nr))//'/'//trim(datestr)//'/'
      else
        dirflexrec=trim(dirflexinit)//trim(recname(nr))//'/'//trim(datestr)//'/'
      endif

      filename=trim(dirflexrec)//'header'
      call readheader(filename,numpoint,outheight)

      nobs=numpoint/numrel

      ! read grid_init files
      allocate(init(numxgrid,numygrid,numzgrid,numpoint))
      call readinit(dirflexrec,numpoint,init)

      ! read initial mixing ratios and convert to flexpart grid  
      allocate(immr(numxgrid,numygrid,numzgrid,ntmax))
      allocate(immrtime(ntmax))
      call initial(outheight,ntmax,numpoint,jd,immr,immrtime)

      ! convert from ppt to output unit for multiplication with immr 
      init=init*coef

      ! scale initial mixing ratios to adjust background
!      immr=immr*0.99

      ! calculate background contribution to mixing ratio
      allocate(yini(numpoint))
      yini(:)=0.

      do nt=1,numpoint
        bkgndvec(:)=0.
        ! calculate time index to initial mixing ratio
        yyyy=yyyymmddhh/1000000
        mm=(yyyymmddhh-1000000*yyyy)/10000
        dd=(yyyymmddhh-1000000*yyyy-10000*mm)/100
        hh=(yyyymmddhh-yyyy*1000000-mm*10000-dd*100)
        ! subtract length of back trajectory
        jtime=float(yyyy)+float(mm-1)/12.+float(dd-1)/365.+float(hh)/(365.*24.)+float(nt*relperiod)/(365.*24.)-(trajlen/365.)
        it=minloc(abs(immrtime(1:ntini)-jtime),dim=1,mask=abs(immrtime(1:ntini)-jtime).eq.minval(abs(immrtime(1:ntini)-jtime)))
        do kz=1,numzgrid
          do jy=1,numygrid
            do ix=1,numxgrid
              if(isnan(immr(ix,jy,kz,it))) go to 10 
              bkgndvec((jy-1)*numxgrid+ix)=bkgndvec((jy-1)*numxgrid+ix)+init(ix,jy,kz,nt)*immr(ix,jy,kz,it)
              yini(nt)=yini(nt)+init(ix,jy,kz,nt)*immr(ix,jy,kz,it)
10            continue
            end do
          end do
        end do
        ! convert background vector to coarse resolution
        bkgnd(indm+nt,nr,:)=matmul(real(gridcrs),bkgndvec)
      end do ! numpoint
  
      ! average background to observation time step

      do nt=1,nobs
        bkgnd_obs((ind+nt-1),nr,:)=sum(bkgnd(indm+(nt-1)*numrel+1:indm+nt*numrel,nr,:),dim=1)/float(numrel)
        allobs((ind+nt-1),nr,6)=real(sum(yini((nt-1)*numrel+1:nt*numrel))/float(numrel),kind=8)
      end do

      jd=jd+eomday
      ind=ind+nobs
      indm=indm+numpoint

      deallocate(init)
      deallocate(immr)
      deallocate(immrtime)
      deallocate(yini)

    end do ! jd
  end do  ! nr


end subroutine backgroundini  

