!---------------------------------------------------------------------------------------
! FLEXINVERT : bayesian1
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Finds the solution vector x corresponding to the probability maximum, that is
!! the solution which minimizes the cost function:
!!
!! J(x) = (x-xb)^T*B^(-1)*(x-xb) + (Hx-y)^T*R^(-1)*(Hx-y)
!!
!! where:
!!   xb = prior/background value of x (n)
!!   B  = prior error covariance matrix (nxn)
!!   H  = kernel matrix (mxn)
!!   R  = observation error covariance matrix (mxm)
!!   y  = observation vector (m)
!!
!! Optimal value of x:
!!
!!   x = xb + BH^T(HBH^T + R)^(-1)(y - Hxb)
!!
!! and its covariance:
!!
!!   cova = B - BH^T(HBH^T + R)^(-1)HB
!!
!! Ref: Tarantola, Inverse Problem Theory, 2005
!---------------------------------------------------------------------------------------

subroutine bayesian1(nn,nm,covb,covr,yobs,kern,xb,x,cova,gain)

  use mod_var

  implicit none

  integer :: i,j,nn,nm,N,lda,info
  real(kind=8), dimension(nn,nn) :: covb
  real, dimension(nn,nn) :: cova
  real, dimension(nm,nm) :: covr
  real, dimension(nm) :: yobs,ymod,yerr
  real, dimension(nm,nn) :: kern
  real, dimension(nn,nm) :: kernt
  real, dimension(nn) :: x,xb
  real(kind=8), dimension(nm,nn) :: kernb
  real(kind=8), dimension(nn,nm) :: bkernt,gain
  real(kind=8), dimension(nm,nm) :: M,mkern,imkern

  write(logid,*) 'Performing the inversion step...'

  ! calculate y - Hxb
  ymod(:)=0.
  yerr(:)=0.
  do i=1,nm
    ymod(i)=dot_product(kern(i,:),xb)
    yerr(i)=yobs(i)-ymod(i)
  end do
  
  ! calculate H^T
  kernt=transpose(kern)

  ! calculate BH^T
  bkernt=matmul(covb,dble(kernt))

  ! calculate (HBH^T + R)
  mkern=matmul(kern,bkernt)+dble(covr)

  ! calculate condition number of (HBH^T + R)
  call calcconditnum(nm,mkern)

  ! calculate (HBH^T + R)^-1
  ! use Cholesky factorization
  N=nm            ! order of (HBH^T + R)
  M=mkern         ! symmetric matrix
  lda=nm          ! leading dimension of (HBH^T + R)
  call dpotrf('L',N,M,lda,info)
  if(info.gt.0) write(logid,*) 'Error in calculating lower Cholesky triangle'
  call dpotri('L',N,M,lda,info)
  if(info.gt.0) write(logid,*) 'Error in inverting (HBH^T + R)'
  ! returns lower ('L') triangle of inverse of M
  imkern=M
  do i=1,nm
    do j=1,i
      imkern(j,i)=M(i,j)
    end do
  end do

  ! calculate Gain matrix BH^T(HBH^T + R)^(-1)
  gain=matmul(bkernt,imkern)

  ! calculate x = xb + BH^T(HBH^T + R)^(-1)(y-Hxb)
  x(:)=0.
  do i=1,nn
    x(i)=xb(i)+dot_product(real(gain(i,:),kind=4),yerr)
  end do

  ! calculate posterior covariance: B - BH^T(HBH^T + R)^(-1)HB
  kernb=matmul(dble(kern),covb)
  cova=real(covb-matmul(gain,kernb),kind=4)

  write(logid,*) 'Congratulations you have optimized the control vector'

end subroutine bayesian1
