!---------------------------------------------------------------------------------------
! FLEXINVERT : bayesian2
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Finds the solution vector x corresponding to the probability maximum, that is
!! the solution which minimizes the cost function:
!!
!! J(x) = (x-xb)^T*B^(-1)*(x-xb) + (Hx-y)^T*R^(-1)*(Hx-y)
!!
!! where:
!!   xb = prior/background value of x (n)
!!   B  = prior error covariance matrix (nxn)
!!   H  = kernel matrix (mxn)
!!   R  = observation error covariance matrix (mxm)
!!   y  = observation vector (m)
!!
!! Optimal value of x:
!! only inverting symmetric matrices B, R and (H^TR^(-1)H + B^(-1)) 
!!
!!   x = xb + (H^TR^(-1)H + B^(-1))^(-1)H^TR^(-1)(y-Hxb)
!!
!! and its covariance:
!!
!!   cova = (H^TR^(-1)H + B^(-1))^(-1)
!!
!! Ref: Tarantola, Inverse Problem Theory, 2005
!---------------------------------------------------------------------------------------

subroutine bayesian2(nn,nm,covb,covr,yobs,kern,xb,x,cova,gain)

  use mod_var

  implicit none

  integer :: i,j,nn,nm,N,M,lda,info
  integer :: ldb,ldu,ldvt,lwork
  integer, dimension(:), allocatable :: iwork
  real(kind=8), dimension(:), allocatable :: work
  real, dimension(nn,nn) :: cova
  real(kind=8), dimension(nn,nn) :: covb,B,icovb,akern,A,iakern
  real(kind=8), dimension(nn,nn) :: U,UT,V,VT,S,iS,ViS
  real, dimension(nm,nm) :: covr
  real(kind=8), dimension(nm,nm) :: R,icovr
  real, dimension(nm) :: yobs,ymod,yerr
  real, dimension(nm,nn) :: kern
  real, dimension(nn,nm) :: kernt
  real(kind=8), dimension(nn,nm) :: lhs,gain
  real, dimension(nn) :: x,xb
  real(kind=8), dimension(nn) :: sigma

  write(logid,*) 'Performing the inversion step...'

  ! calculate y - Hxb
  ymod(:)=0.
  yerr(:)=0.
  do i=1,nm
    ymod(i)=dot_product(kern(i,:),xb)
    yerr(i)=yobs(i)-ymod(i)
  end do
  
  ! calculate H^T
  kernt=transpose(kern)

  ! calculate R^(-1)
  ! use Cholesky factorization 
  N=nm           ! order of R
  R=dble(covr)   ! symmetric matrix
  lda=nm         ! leading dimension of R
  call dpotrf('L',N,R,lda,info)
  call dpotri('L',N,R,lda,info)
  if(info.gt.0) write(logid,*) 'Error in inverting R'
  ! returns lower ('L') triangle of inverse of R
  icovr=R
  do i=1,nm
    do j=1,i 
      icovr(j,i)=R(i,j)
    end do
  end do

  ! calculate B^(-1)
  ! use SVD (Cholesky fails for non-positive-definite matrices)
  M=nn           ! number of rows of B
  N=nn           ! number of columns of B
  B=covb         ! matrix of dimension M x N
  ldb=M          ! leading dimension of B
  ldu=M          ! leading dimension of U
  ldvt=N         ! leading dimension of VT
  lwork=3*min(M,N)+max(max(M,N),4*min(M,N)*min(M,N)+4*min(M,N))
  allocate(work(lwork))
  allocate(iwork(8*min(M,N)))
  call dgesdd('A',M,N,B,ldb,sigma,U,ldu,VT,ldvt,work,lwork,iwork,info)
  if(info.ne.0) then
    print*, 'Error in singular value decomposition'
    if(info.lt.0) print*, 'the argument in position ',info,' had an illegal value'
    if(info.gt.0) print*, 'the algorithm failed to converge'
    stop
  endif
  ! calculate B^(-1)
  UT=transpose(U)
  V=transpose(VT)
  S(:,:)=0.
  iS(:,:)=0.
  do i=1,nn
    if(sigma(i).ne.0.) iS(i,i)=1./sigma(i)
    S(i,i)=sigma(i)
  end do
  ViS=matmul(V,iS)
  icovb=matmul(ViS,UT)
  deallocate(iwork)
  deallocate(work)

  ! calculate (H^TR^(-1)H + B^(-1))
  lhs=matmul(dble(kernt),icovr)
  akern=matmul(lhs,dble(kern))+icovb

  ! calculate condition number of (H^TR^(-1)H + B^(-1))
  call calcconditnum(nn,akern)

  ! calculate inverse of (H^TR^(-1)H + B^(-1))
  ! use Cholesky factorization
  N=nn           ! order of (H^TR^(-1)H + B^(-1))
  A=akern        ! symmetric matrix
  lda=nn         ! leading dimension of (H^TR^(-1)H + B^(-1))
  call dpotrf('L',N,A,lda,info)
  if(info.gt.0) write(logid,*) 'Error in calculating lower Cholesky triangle'
  call dpotri('L',N,A,lda,info)
  if(info.gt.0) write(logid,*) 'Error in inverting (H^TR^(-1)H + B^(-1))'
  ! returns lower ('L') triangle of inverse of A
  iakern=A
  do i=1,nn
    do j=1,i
      iakern(j,i)=A(i,j)
    end do
  end do

  ! calculate Gain matrix (H^TR^(-1)H + B^(-1))^(-1)H^TR^(-1)
  gain=matmul(iakern,lhs)

  ! calculate x = xb + G(y - Hxb)
  x(:)=0.
  do i=1,nn
    x(i)=xb(i)+dot_product(real(gain(i,:),kind=4),yerr)
  end do

  ! posterior covariance matrix
  cova=real(iakern,kind=4)

  write(logid,*) 'Congratulations you have optimized the control vector'

end subroutine bayesian2
