!---------------------------------------------------------------------------------------
! FLEXINVERT : calcconditnum
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates the condition number of any real symmetric matrix A
!! The condition number of A is the ratio of the largest to the smallest eigenvalue
!! of A.
!---------------------------------------------------------------------------------------

subroutine calcconditnum(nn,mat)

  use mod_var

  implicit none

  integer :: nn,N,M,il,iu,lda,ldz,lwork,liwork,info
  real(kind=8) :: vl,vu,abstol
  integer, dimension(:), allocatable :: isuppz
  real(kind=8), dimension(:), allocatable :: work
  real(kind=8), dimension(:), allocatable :: W
  real(kind=8), dimension(:,:), allocatable :: Z
  integer, dimension(:), allocatable :: iwork
  real(kind=8), dimension(nn,nn) :: mat,A
  real :: condnum

  ! calculate eigenvalue decomposition
  N=nn            ! order of A
  lda=nn          ! leading dimension of A
  ldz=nn          ! leading dimension of Z
  A=mat           ! symmetric matrix
  vl=1.           ! lower bound of interval to search for eigenvalues (not used if range='A' or 'I')
  vu=1.           ! upper bound of interval to search for eigenvalues (not used if range='A' or 'I')
  il=1            ! indice of smallest eignenvalue returned (not used if range='A' or 'I')
  iu=1            ! indice of largest eignenvalue returned (not used if range='A' or 'I')
  abstol=0.       ! absolute error tolerance (if abstol<=0 uses EPS*norm(T) see LAPACK info)
  lwork=26*N      ! dimension of work 
  liwork=10*N     ! dimension of iwork
  allocate(work(lwork))
  allocate(iwork(liwork))
  allocate(isuppz(2*N))
  allocate(Z(N,N))
  allocate(W(N))
  call dsyevr('V','A','L',N,A,lda,vl,vu,il,iu,abstol,M,W,Z,ldz,isuppz,work,lwork,iwork,liwork,info)
  if(info.ne.0) then
    write(logid,*) 'Error in eigenvalue decomposition'
    if(info.gt.0) write(logid,*) 'Internal error'
    if(info.lt.0) write(logid,fmt='(1X,A25,I5,A21)') 'The argument in position ',info,' had an illegal value'
  endif
  deallocate(work)
  deallocate(iwork)
  deallocate(isuppz)
  deallocate(Z)

  ! note: eigenvalues are sorted in ascending order
  condnum=W(nn)/W(1)
  deallocate(W)

  write(logid,fmt='(1X,A18,F11.3)') 'Condition number: ',condnum

end subroutine calcconditnum

