!---------------------------------------------------------------------------------------
! FLEXINVERT : calccost
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates the cost of the Bayesian function:
!! J(x) = (x-xb)^T*B^(-1)*(x-xb) + (Hx-y)^T*R^(-1)*(Hx-y)
!!
!! Options:
!!   stat: status 1 = a priori, 2 = a posteriori
!---------------------------------------------------------------------------------------

subroutine calccost(stat,nn,nm,x,xb,yobs,kern,covr,covb,cost)

  use mod_var

  implicit none

  integer :: i,j,stat,nn,nm,M,N,lda,info
  integer :: ldb,ldu,ldvt,lwork
  integer, dimension(:), allocatable :: iwork
  real(kind=8), dimension(:), allocatable :: work
  real(kind=8), dimension(nn) :: sigma
  real, dimension(nn) :: x,xb,xerr,lhsb
  real, dimension(nm) :: yobs,yerr,lhsr
  real, dimension(nm,nn) :: kern
  real, dimension(nm,nm) :: covr
  real(kind=8), dimension(nm,nm) :: R,icovr
  real(kind=8), dimension(nn,nn) :: covb,B,icovb
  real(kind=8), dimension(nn,nn) :: U,UT,V,VT,S,iS,ViS
  real, dimension(nm,1) :: ymat
  real, dimension(nn,1) :: xmat
  real, dimension(1) :: costr,costb,cost

  ! calculate R^(-1)
  ! use Cholesky factorization 
  N=nm         ! order of R
  R=dble(covr) ! symmetric matrix
  lda=nm       ! leading dimension of R
  call dpotrf('L',N,R,lda,info)
  call dpotri('L',N,R,lda,info)
  if(info.gt.0) write(logid,*) 'Error in inverting R'
  ! returns lower ('L') triangle of inverse of R
  icovr=R
  do i=1,nm
    do j=1,i
      icovr(j,i)=R(i,j)
    end do
  end do

  ! calculate B^(-1)
  ! use SVD (Cholesky fails for non-positive-definite matrices)
  M=nn           ! number of rows of B
  N=nn           ! number of columns of B
  B=covb         ! matrix of dimension M x N
  ldb=M          ! leading dimension of B
  ldu=M          ! leading dimension of U
  ldvt=N         ! leading dimension of VT
  info=0
  lwork=3*min(M,N)+max(max(M,N),4*min(M,N)*min(M,N)+4*min(M,N))
  allocate(work(lwork))
  allocate(iwork(8*min(M,N)))
  call dgesdd('A',M,N,B,ldb,sigma,U,ldu,VT,ldvt,work,lwork,iwork,info)
  if(info.ne.0) then
    print*, 'Error in singular value decomposition'
    if(info.lt.0) print*, 'the argument in position ',info,' had an illegal value'
    if(info.gt.0) print*, 'the algorithm failed to converge'
    stop
  endif
  ! calculate B^(-1)
  UT=transpose(U)
  V=transpose(VT)
  S(:,:)=0.
  iS(:,:)=0.
  do i=1,nn
    if(sigma(i).ne.0.) iS(i,i)=1./sigma(i)
    S(i,i)=sigma(i)
  end do
  ViS=matmul(V,iS)
  icovb=matmul(ViS,UT)
  deallocate(iwork)
  deallocate(work)

  ! cost a priori
  ! J = (Hxb-y)^T*R^(-1)*(Hxb-y)
  if(stat.eq.1) then
    yerr=matmul(kern,xb)-yobs
    lhsr=matmul(yerr,real(icovr,kind=4))
    ymat=reshape(yerr,(/nm,1/))
    cost=matmul(lhsr,ymat)
    write(logid,fmt='(1X,A20,F11.3)') 'Cost a priori (Jy): ',cost
  endif

  ! cost a posteriori
  ! J = (x-xb)^T*B^(-1)*(x-xb) + (Hx-y)^T*R^(-1)*(Hx-y)
  if(stat.eq.2) then
    yerr=matmul(kern,x)-yobs
    lhsr=matmul(yerr,real(icovr,kind=4))
    ymat=reshape(yerr,(/nm,1/))
    costr=matmul(lhsr,ymat)
    xerr=x-xb
    lhsb=matmul(xerr,real(icovb,kind=4))
    xmat=reshape(xerr,(/nn,1/))
    costb=matmul(lhsb,xmat)
    cost=costb+costr
    write(logid,fmt='(1X,A24,F11.3)') 'Cost a posteriori (Jy): ',costr
    write(logid,fmt='(1X,A24,F11.3)') 'Cost a posteriori (Jx): ',costb
    write(logid,fmt='(1X,A29,F11.3)') 'Cost a posteriori total (J): ',cost
    write(logid,fmt='(1X,A36,F11.3)') 'Degrees of freedom of signal (DFS): ',costb
  endif

end subroutine calccost



