!---------------------------------------------------------------------------------------
! FLEXINVERT : calcperturb
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Adds random perturbation to the control vector (prior emissions) and observations
!! with the same error covariance statistics as the B and R matrices, respectively
!!
!! xb' = xb + B^0.5*rb
!!
!! y' = y + R^0.5*rr
!!
!! where rb and rr are random vectors of same length as xb and y, respectively
!---------------------------------------------------------------------------------------

subroutine calcperturb(nn,nm,covb,covr,xb,yobs,ymod,outheight)

  use mod_var

  implicit none

  integer :: i,ind,nn,nm,ierr,N,M,lda,ldz,il,iu,lwork,liwork,info
  character(len=5) :: seedstr
  real, dimension(nzmax) :: outheight
  real, dimension(nn) :: xb,rbvec
  real, dimension(nm) :: yobs,rrvec,ymod
  real(kind=8), dimension(nn,nn) :: covb,sqrtcovb
  real(kind=8), dimension(nn,nn) :: A
  real(kind=8) :: vl,vu,abstol
  real, dimension(nm,nm) :: covr
  real(kind=8), dimension(nm,nm) :: sqrtcovr
  real(kind=8),dimension(nm,nm) :: U
  real :: ri
  integer, dimension(:), allocatable :: isuppz
  real(kind=8), dimension(:), allocatable :: work
  real(kind=8), dimension(:,:), allocatable :: sqrtw
  real(kind=8), dimension(:), allocatable :: W
  real(kind=8), dimension(:,:), allocatable :: Z
  integer, dimension(:), allocatable :: iwork
  integer :: ssize
  integer, dimension(:), allocatable :: seedvec


  write(logid,*) 'Adding random perturbation to xb and yobs'

  ! record true emissions before perturbation

  open(100,file=trim(dirinvert)//'true.txt',status='replace',action='write',iostat=ierr)
  do i=1,nbox*nemsteps
    write(100,*) xb(i)*outheight(1)/numscale
  end do
  do i=1,nbkgnd
    write(100,*) xb(nbox*nemsteps+i)
  end do
  close(100)

  ! set yobs equal to ymod (forward model before perturbing xb)

  yobs=ymod

  ! generate vector of random numbers of length equal to xb
  
  call random_seed
  call random_seed(size=ssize)
  allocate(seedvec(ssize))
  do i=1,ssize
    seedvec(i)=seed
  end do
  call random_seed(put=seedvec(1:ssize))
  do i=1,nn
    call random_number(ri)
    rbvec(i)=ri
  end do
  do i=1,nm
    call random_number(ri)
    rrvec(i)=ri
  end do
  deallocate(seedvec)

  ! normalize random numbers and subtract 1 to have mean of zero
  rrvec=rrvec*nm/sum(rrvec)-1.
  rbvec=rbvec*nn/sum(rbvec)-1.

  write(seedstr,'(I3)') seed
  open(100,file=trim(dirinvert)//'rand'//trim(seedstr)//'.txt',action='write',status='replace',iostat=ierr)
  do i=1,nn
    write(100,fmt='(F6.4)') anint(rbvec(i)*10000.)/10000.
  end do
  close(100)

  ! calculate perturbation B^(0.5)*r

  ! calculate B^(0.5) by first computing eigenvalue decomposition
  N=nn            ! order of A
  lda=nn          ! leading dimension of A
  ldz=nn          ! leading dimension of Z
  A=covb          ! symmetric matrix
  vl=1.           ! lower bound of interval to search for eigenvalues (not used if range='A' or 'I')
  vu=1.           ! upper bound of interval to search for eigenvalues (not used if range='A' or 'I')
  il=1            ! indice of smallest eignenvalue returned (not used if range='A' or 'I')
  iu=1            ! indice of largest eignenvalue returned (not used if range='A' or 'I')
  abstol=0.       ! absolute error tolerance (if abstol<=0 uses EPS*norm(T) see LAPACK info)
  lwork=26*N      ! dimension of work 
  liwork=10*N     ! dimension of iwork
  allocate(work(lwork))
  allocate(iwork(liwork))
  allocate(isuppz(2*N))
  allocate(Z(N,N))
  allocate(sqrtw(N,N))
  allocate(W(N))
  call dsyevr('V','A','L',N,A,lda,vl,vu,il,iu,abstol,M,W,Z,ldz,isuppz,work,lwork,iwork,liwork,info)
  if(info.ne.0) then
    write(logid,*) 'Error in eigenvalue decomposition'
    if(info.gt.0) write(logid,*) 'Internal error'
    if(info.lt.0) write(logid,fmt='(1X,A25,I5,A21)') 'The argument in position ',info,' had an illegal value'
  endif
  deallocate(work)
  deallocate(iwork)
  deallocate(isuppz)

  sqrtw(:,:)=0.
  ind=minloc(W,dim=1,mask=W.gt.0.)
 
  ! note: eigenvalues are sorted in ascending order
  do i=ind,N
    sqrtw(i,i)=sqrt(W(i))
  end do
  ! B^(1/2) = Q*L^(1/2)*Q^T
  sqrtcovb(:,ind:N)=matmul(Z(:,ind:N),sqrtw(ind:N,ind:N))
  sqrtcovb=matmul(sqrtcovb(:,ind:N),transpose(Z(:,ind:N)))
  deallocate(sqrtw)
  deallocate(Z)
  deallocate(W)

  xb=xb+matmul(real(sqrtcovb,kind=4),rbvec)

  ! calculate perturbation of R^(0.5)*r

  ! calculate R^(0.5) by first computing eigenvalue decomposition

  N=nm
  lda=nm
  ldz=nm
  U=dble(covr)
  vl=1.
  vu=1.
  il=1
  iu=1
  abstol=0.
  lwork=26*N
  liwork=10*N
  allocate(work(lwork))
  allocate(iwork(liwork))
  allocate(isuppz(2*N))
  allocate(Z(N,N))
  allocate(sqrtw(N,N))
  allocate(W(N))
  call dsyevr('V','A','L',N,U,lda,vl,vu,il,iu,abstol,M,W,Z,ldz,isuppz,work,lwork,iwork,liwork,info)
  if(info.ne.0) then
    write(logid,*) 'Error in eigenvalue decomposition'
    if(info.gt.0) write(logid,*) 'Internal error'
    if(info.lt.0) write(logid,fmt='(1X,A25,I5,A21)') 'The argument in position ',info,' had an illegal value'
  endif
  deallocate(work)
  deallocate(iwork)
  deallocate(isuppz)

  ind=minloc(W,dim=1,mask=W.gt.0.)
  sqrtw(:,:)=0.
  do i=ind,nm
    sqrtw(i,i)=sqrt(W(i))
  end do
  sqrtcovr(:,ind:nm)=matmul(Z(:,ind:nm),sqrtw(ind:nm,ind:nm))
  sqrtcovr=matmul(sqrtcovr(:,ind:nm),transpose(Z(:,ind:nm)))
  deallocate(sqrtw)
  deallocate(Z)
  deallocate(W)

  yobs=yobs+matmul(real(sqrtcovr,kind=4),rrvec)


end subroutine calcperturb


