!---------------------------------------------------------------------------------------
! FLEXINVERT : calcprior
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates prior modelled mixing ratios and observation error and selects
!! rows of the kernel matrix for which there are observations 
!!
!! observation error is composed of:
!! 1) sd of observations over averaging interval
!! 2) model uncertainty (in flexpart run)
!! 3) measurement error (set in control.def)
!! 4) background error (set in control.def)
!---------------------------------------------------------------------------------------

subroutine calcprior(knest_obs,knest,knest_finobs,knest_fin,kout_obs,&
                     &bkgnd_obs,xb,allobs,yobs,ymod,ybgi,yerr,ydat,yrec,nobs)

  use mod_var
  use mod_dates

  implicit none

  integer :: i,j,k,nobs,ierr
  real, dimension(nbox*nemsteps+nbkgnd) :: xb
  real(kind=8), dimension(nobsteps,numrec,6) :: allobs
  real, dimension(nobsteps*numrec) :: yobs,ymod,ybgi,yerr,ytest
  integer, dimension(nobsteps*numrec) :: ydat
  character(len=7), dimension(nobsteps*numrec) :: yrec
  real, dimension(nobsteps*numrec,nemsteps*nbox+nbkgnd) :: knest
  real, dimension(nobsteps*numrec,nemsteps*nbox) :: knest_obs
  real, dimension(nobsteps,numrec,ncrs) :: bkgnd_obs
  real, dimension(nobsteps*numrec,ncrs) :: kout_obs
  real, dimension(nobsteps*numrec,numxnest*numynest) :: knest_fin,knest_finobs
  real, dimension(nlatbin) :: clatbin

  write(logid,*) 'Calculating prior model mixing ratios...'

  knest(:,:)=0.
  knest_fin(:,:)=0.

  if(optimizbg.and.bgmethod.eq.1) then
    do i=1,(nlatbin-1)
      clatbin(i)=0.5*(latbin(i)+latbin(i+1))
    enddo
    clatbin(nlatbin)=0.5*(latbin(nlatbin)+90)
  endif

  nobs=0
  do i=1,nobsteps
    do j=1,numrec
      if(.not.checkprior) then
        if(allobs(i,j,2).eq.0..or.allobs(i,j,6).eq.0.) go to 50
      endif
      if(.not.optimizbg) then
        ! subtract background from observations
        ! if observation lower than background cut observation (passive tracer no sink)
        if((allobs(i,j,2)-allobs(i,j,6)).lt.0.) go to 50
        nobs=nobs+1
        if(bgmethod.eq.2) then
          yobs(nobs)=real(allobs(i,j,2)-allobs(i,j,6),kind=4)-sum(kout_obs((i-1)*numrec+j,:))
          ybgi(nobs)=real(allobs(i,j,6),kind=4)+sum(kout_obs((i-1)*numrec+j,:))
        else
          yobs(nobs)=real(allobs(i,j,2)-allobs(i,j,6),kind=4)
          ybgi(nobs)=real(allobs(i,j,6),kind=4)
        endif 
        ymod(nobs)=dot_product(knest_obs((i-1)*numrec+j,:),xb(:))
        knest(nobs,:)=knest_obs((i-1)*numrec+j,:)
      else
        ! no substraction of background
        nobs=nobs+1
        yobs(nobs)=real(allobs(i,j,2),kind=4)
        knest(nobs,1:nbox*nemsteps)=knest_obs((i-1)*numrec+j,:)
        ! calculate part of kernel matrix for background optimization
        if(bgmethod.eq.2) then
          knest(nobs,nbox*nemsteps+1:nbox*nemsteps+nbkgnd)=kout_obs((i-1)*numrec+j,:)+bkgnd_obs(i,j,:) 
          ymod(nobs)=dot_product(knest(nobs,:),xb(:))
          ybgi(nobs)=dot_product(knest(nobs,nbox*nemsteps+1:nbox*nemsteps+nbkgnd),xb(nbox*nemsteps+1:nbox*nemsteps+nbkgnd))
!          ytest(nobs)=real(allobs(i,j,6),kind=4)+sum(kout_obs((i-1)*numrec+j,:))
        else
          k=minloc(abs(clatbin(:)-reclat(j)),dim=1,mask=abs(clatbin(:)-reclat(j)).eq.minval(abs(clatbin(:)-reclat(j))))
          knest(nobs,nbox*nemsteps+k)=allobs(i,j,6)
          ymod(nobs)=dot_product(knest(nobs,:),xb(:))
          ybgi(nobs)=real(allobs(i,j,6),kind=4)
        endif        
      endif
      ! quadratic sum of observation sd, observation, model and background errors
      yerr(nobs)=real(sqrt(allobs(i,j,3)**2+allobs(i,j,4)**2+allobs(i,j,5)**2+bkgerr**2),kind=4)
      knest_fin(nobs,:)=knest_finobs((i-1)*numrec+j,:)
      ydat(nobs)=caldate(allobs(i,j,1))
      yrec(nobs)=recname(j)
50    continue
    end do
  end do
  write(logid,fmt='(1X,A29,I5)'), 'Total number of observations:',nobs

!  if(optimizbg.and.bgmethod.eq.2) then
!    open(400,file=trim(dirinvert)//'testbg.txt',status='replace',action='write',iostat=ierr)
!    do i=1,nobs
!      write(400,fmt='(F9.2,1X,F9.2)') ytest(i),ybgi(i)
!    end do
!    close(400)
!  endif

end subroutine calcprior
