!---------------------------------------------------------------------------------------
! FLEXINVERT : calcsrmatrix
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates the source-receptor matrix describing the relationship between 
!! surface fluxes and the observations at a given receptor for the nested domain.
!! Reads the surface influence grid_time files for each station and timestep and
!! allocates these to the corresponding row and columns of the source-receptor matrix.
!! Also calculates contribution from each grid-cell to the background mixing ratio 
!! for each observation and converts this to the coarse grid resolution.
!---------------------------------------------------------------------------------------

subroutine calcsrmatrix(knest,knest_fin,kout,moderr)

  use mod_var
  use mod_dates

  implicit none

  character(len=100) :: dirflexrec,filename,datestr
  integer :: inde,indm,cnt,cntp,numpoint
  integer :: nr,nt,k,ix,jy
  logical :: nextmonth,nextemstep
  integer :: eomday
  integer :: yyyymmddhh,yyyymm,preyyyymm,yyyy,mm
  real(kind=8) :: jd,jdi,jdf
  real, dimension(numrec*(numday*24/relperiod),nemsteps*nbox) :: knest
  real, dimension(numrec*(numday*24/relperiod),numxnest*numynest) :: knest_fin
  real, dimension(numrec*(numday*24/relperiod),ncrs) :: kout
  real, dimension(numday*24/relperiod,numrec) :: moderr
  real, dimension(nzmax) :: outheight
  real, dimension(:,:,:,:), allocatable :: grid,mgrid,mgridp,uncert,muncert
  real, dimension(:), allocatable :: surfinfvec,surfinfvecp,surfinfvecfin,bkgndinf,bkgndinfcrs
  real, dimension(:), allocatable :: mgridvec,mgridvecp
  real :: err,jtime

  allocate(grid(numxgrid,numygrid,numzgrid,maxpoint))
  allocate(mgrid(numxgrid,numygrid,numzgrid,maxpoint))
  allocate(mgridp(numxgrid,numygrid,numzgrid,maxpoint))  
  allocate(uncert(numxgrid,numygrid,numzgrid,maxpoint))
  allocate(muncert(numxgrid,numygrid,numzgrid,maxpoint))

  allocate(surfinfvec(nbox))
  allocate(surfinfvecp(nbox))
  allocate(surfinfvecfin(numxnest*numynest))
  allocate(bkgndinf(numxgrid*numygrid))
  allocate(bkgndinfcrs(ncrs))
  allocate(mgridvec(numxgrid*numygrid))
  allocate(mgridvecp(numxgrid*numygrid))

  mgrid(:,:,:,:)=0.
  mgridp(:,:,:,:)=0.

  write(logid,fmt='(1X,A29,I6,1X,I6)') 'Dimensions of knest matrix: ',shape(knest)
  write(logid,*) 'Building knest and kout matrices...'

  ! loop over receptors
  do nr=1,numrec
    jd=juldatei
    jdi=juldatei
    inde=0
    indm=0
    nextmonth=.true.
    nextemstep=.false.
    write(logid,*) 'Reading grids for receptor: '//trim(recname(nr))
    ! loop over trajectories month by month and allocate surface influence
    ! to relevant emission timestep
    do while (jd.le.juldatef)
      yyyymmddhh=caldate(jd)
      yyyymm=yyyymmddhh/10000
      preyyyymm=yyyymm
      write(logid,fmt='(1X,A24,1X,I6)') 'Reading grids for month:',preyyyymm
      write(datestr,'(i6)') yyyymm
      dirflexrec=trim(dirflex)//trim(recname(nr))//'/'//trim(datestr)//'/'
      if(usedirflex) then
        filename=trim(dirflexrec)//'header_grid_time'
      else
        filename=trim(dirflexrec)//'header'
      endif
      eomday=calceomday(yyyymm)
      numpoint=eomday*24/relperiod
      call readheader(filename,numpoint,outheight)
      ! if in this directory for first time then adjust
      ! start date for trajectories going back in time
      if(nextmonth) jd=max(juldatei,(jd-trajlen))
      do while (yyyymm.le.preyyyymm)
        do while (jd.lt.(jdi+tstep))
          call readgrid(dirflexrec,jd,numpoint,grid,uncert)
          if(jd.lt.jdi.and.jd.gt.(jdi-tstep)) then
            ! grid belongs to previous timestep
            mgridp=mgridp+grid
            cntp=cntp+1
          elseif(jd.ge.jdi) then
            ! grids belong to current timestep
            mgrid=mgrid+grid
            muncert=muncert+uncert
            cnt=cnt+1
          endif
          jd=jd+float(abs(loutstep))/3600./24.
          yyyymmddhh=caldate(jd)
          yyyymm=yyyymmddhh/10000
          ! if not finished emission timestep but need to move to next month
          if(yyyymm.gt.preyyyymm) then
            nextmonth=.true.
            nextemstep=.false.
            go to 20
          endif
        end do
        ! finished reading grids for current emission timestep
        jdi=jdi+tstep
        nextmonth=.false.
        nextemstep=.true.
        go to 20
      end do ! yyyymm
20    continue
      ! assign knest and kout matrices by looping over trajectories in emission timestep
      ! keep original trajectory time resolution for now
      ! knest and kout organised by receptor (nr), month (indm), trajectory (k)
      do k=1,numpoint
        ! vectorize surface influence and calculate contribution to
        ! background from outside nested domain
        yyyy=yyyymmddhh/1000000
        mm=(yyyymmddhh-yyyy*1000000)/10000
        jtime=float(yyyy)+float(mm-1)/12.+float(k*relperiod/24)/365.
        nt=minloc(abs(timemi(:)-jtime),dim=1,mask=abs(timemi(:)-jtime).eq.minval(abs(timemi(:)-jtime)))
        mgridvec(:)=0.
        mgridvecp(:)=0.
        bkgndinf(:)=0.
        do jy=1,numygrid
          do ix=1,numxgrid
            mgridvec((jy-1)*numxgrid+ix)=mgrid(ix,jy,1,k)
            mgridvecp((jy-1)*numxgrid+ix)=mgridp(ix,jy,1,k)
            ! calculate contribution to change in mixing ratio from outside domain
            bkgndinf((jy-1)*numxgrid+ix)=mgrid(ix,jy,1,k)*emisflex(ix,jy,nt)*area_grid(ix,jy)
          end do
        end do
        ! surface influence for nested domain at fine resolution and for
        ! contribution to background set values inside nest to zero 
        surfinfvecfin(:)=0.
        do jy=1,numynest
          do ix=1,numxnest
            surfinfvecfin((jy-1)*numxnest+ix)=mgrid(ixl+ix-1,iyl+jy-1,1,k)
            bkgndinf((iyl+jy-2)*numxgrid+ixl+ix-1)=0.
          end do
        end do
        ! map surface influence to inversion grid in nested domain
        ! no area weighting of surfinf because emissions already account for area
        surfinfvec=matmul(real(gridoper),mgridvec)
        surfinfvecp=matmul(real(gridoper),mgridvecp)
        ! map contribution to background to coarse grid
        bkgndinfcrs=matmul(real(gridcrs),bkgndinf)
        bkgndinfcrs=bkgndinfcrs/area_crs
        ! write to knest_fin 
        knest_fin((nr-1)*(numday*24/relperiod)+indm+k,:)=&
                           &knest_fin((nr-1)*(numday*24/relperiod)+indm+k,:)+surfinfvecfin
        ! write to knest
        if((inde-1).gt.0.and.cntp.gt.0) knest((nr-1)*(numday*24/relperiod)+indm+k,((inde-1)*nbox+1):inde*nbox)=&
                           &knest((nr-1)*(numday*24/relperiod)+indm+k,((inde-1)*nbox+1):inde*nbox)+surfinfvecp
        if(cnt.gt.0) knest((nr-1)*(numday*24/relperiod)+indm+k,inde*nbox+1:(inde+1)*nbox)=&
                           &knest((nr-1)*(numday*24/relperiod)+indm+k,inde*nbox+1:(inde+1)*nbox)+surfinfvec
        ! write to kout
         kout((nr-1)*(numday*24/relperiod)+indm+k,:)=bkgndinfcrs
        ! calculate model error
        call modelerror(jd,muncert,err)
        moderr(indm+k,nr)=err
      end do ! numpoint 
      if(nextmonth) then
        indm=indm+numpoint
      endif
      if(nextemstep) then
        inde=inde+1
        if(inde.eq.nemsteps) go to 30
      endif
      mgridp(:,:,:,:)=0.
      mgrid(:,:,:,:)=0.
      muncert(:,:,:,:)=0.
      cntp=0
      cnt=0
    end do ! juldatef
30  continue
    mgridp(:,:,:,:)=0.
    mgrid(:,:,:,:)=0.
    muncert(:,:,:,:)=0.
    cntp=0
    cnt=0
  end do ! numrec

  ! mean over nemsteps
  knest_fin=knest_fin/nemsteps

  ! convert from ppt mass mixing ratio to volume mixing ratio
  knest=knest*coef*28.97/molarmass
  knest_fin=knest_fin*coef*28.97/molarmass
  kout=kout*coef*28.97/molarmass
  moderr=moderr*coef*28.97/molarmass

  deallocate(grid)
  deallocate(mgrid)
  deallocate(mgridp)
  deallocate(uncert)
  deallocate(muncert)
  deallocate(surfinfvec)
  deallocate(surfinfvecp)
  deallocate(surfinfvecfin)
  deallocate(mgridvec)
  deallocate(mgridvecp)
  deallocate(bkgndinf)
  deallocate(bkgndinfcrs)

end subroutine calcsrmatrix

