!---------------------------------------------------------------------------------------
! FLEXINVERT : convertgrid
!---------------------------------------------------------------------------------------
!> Converts 2D gridded data (gridin) to flexpart grid (gridflex) 
!! FLEXPART grid: numxgrid,numygrid,dxout,dyout,outlon0,outlat0
!! INPUT grid: lon,lat
!! options:
!!  - check: check total mass of input and output grids (logical)
!!  - units: output grid units 1=flux, 2=mass, 3=concentration 
!!  - midpoints: input grid coordinates are midpoints or west/south edges (logical)
!!
!! note: currently uses global midpoints logical
!---------------------------------------------------------------------------------------

subroutine convertgrid(check,units,outheight,nlon,nlat,lon,lat,ntime,gridin,gridflex)

  use mod_var

  implicit none

  integer :: units,nlon,nlat,ntime
  logical :: check
  real, dimension(nlon) :: lon,dlon,lonl,lonr,lonm
  real, dimension(nlat) :: lat,dlat,latl,latr,latm
  real, dimension(nlon,nlat,ntime) :: gridin
  real, dimension(nzmax) :: outheight
  integer :: nt,ix,jy,ix1,ix2,jy1,jy2,ii,jj
  real :: xl,xr,xm,yl,yr,ym,xolap1,xolap2,yolap1,yolap2
  real :: mass,aolap,aolaptot,outtot,intot,area
  real, dimension(numxgrid,numygrid,ntime) :: gridflex

  gridflex(:,:,:)=0.
  outtot=0.
  intot=0.

  do ix=2,nlon
    dlon(ix)=abs(lon(ix)-lon(ix-1))
  end do
  dlon(1)=dlon(2)
  do jy=2,nlat
    dlat(jy)=abs(lat(jy)-lat(jy-1))
  end do
  dlat(1)=dlat(2)

  ! if lat and lon coordinates for gridin are midpoints
  ! calculate west/east(l/r) and south/north(l/r) edges
  ! otherwise calculate midpoints
  if(midpoints) then
    do ix=1,nlon
      lonl(ix)=lon(ix)-0.5*dlon(ix)
      lonr(ix)=lon(ix)+0.5*dlon(ix)
      lonm(ix)=lon(ix)
    end do
    do jy=1,nlat
      latl(jy)=lat(jy)-0.5*dlat(jy)
      latr(jy)=lat(jy)+0.5*dlat(jy)
      latm(jy)=lat(jy)
    end do
  else
    lonl=lon
    do ix=1,nlon
      lonr(ix)=lon(ix)+dlon(ix)
    end do
    do ix=1,nlon
      lonm(ix)=lonl(ix)+0.5*dlon(ix)
    end do
    latl=lat
    do jy=1,nlat
      latr(jy)=lat(jy)+dlat(jy)
    end do
    do jy=1,nlat
      latm(jy)=latl(jy)+0.5*dlat(jy)
    end do
  endif  

  ! loop over time and then y and x dimensions
  do nt=1,ntime
    do jy=1,numygrid
      yl=outlat0+float(jy-1)*dyout
      yr=outlat0+float(jy)*dyout
      ym=0.5*(yl+yr)
      do ix=1,numxgrid
        xl=outlon0+float(ix-1)*dxout
        xr=outlon0+float(ix)*dxout
        xm=0.5*(xl+xr)
        ! check new indices
        ix1=maxloc(lonl,dim=1,mask=lonl.le.xl)
        ix2=maxloc(lonl,dim=1,mask=lonl.le.xr)
        jy1=maxloc(latl,dim=1,mask=latl.le.yl)
        jy2=maxloc(latl,dim=1,mask=latl.le.yr)
        mass=0.
        aolaptot=0.
        if(ix1.eq.0.or.jy1.eq.0) write(logid,*) 'Error: boundaries of output grid not within input grid'
        ! loop over grid cells in in-grid which overlap with the grid cell in out-grid
        do jj=jy1,jy2
          do ii=ix1,ix2
            ! overlapping area has boundaries xolap1,xolap2 and yolap1,yolap2
            xolap1=max(lonl(ii),xl)
            xolap2=min(lonr(ii),xr)
            yolap1=max(latl(jj),yl)
            yolap2=min(latr(jj),yr)
            ! calculate overlapping area
            call gridarea(yolap1,abs(yolap2-yolap1),abs(xolap2-xolap1),aolap)
            mass=mass+gridin(ii,jj,nt)*aolap
            aolaptot=aolaptot+aolap    
          end do
        end do
        if(aolaptot.ne.0) then
          if(units.eq.1) then
            ! area flux (kg/m2/h)
            gridflex(ix,jy,nt)=mass/aolaptot
          else if(units.eq.2) then
            ! mass (kg/h)
            gridflex(ix,jy,nt)=mass
          else if(units.eq.3) then
            ! volume flux (kg/m3/s)
            gridflex(ix,jy,nt)=mass/3600./outheight(1)/aolaptot
          endif
        endif
        outtot=outtot+mass
        ! complete righthand edge of grid
        mass=0.
        aolaptot=0.
        if(ix.eq.numxgrid) then
          xolap1=lonl(ix2)
          xolap2=xr
          do jj=jy1,jy2
            yolap1=max(latl(jj),yl)
            yolap2=min(latr(jj),yr)
            ! calculate overlapping area
            call gridarea(yolap1,abs(yolap2-yolap1),abs(xolap2-xolap1),aolap)
            mass=mass+gridin(ix2,jj,nt)*aolap
            aolaptot=aolaptot+aolap
          end do
          if(aolaptot.ne.0) then
            if(units.eq.1) then
              gridflex(ix,jy,nt)=mass/aolaptot
            else if(units.eq.2) then
              gridflex(ix,jy,nt)=mass
            else if(units.eq.3) then
              gridflex(ix,jy,nt)=mass/3600./outheight(1)/aolaptot
            endif
          endif   
          outtot=outtot+mass
        endif
        ! complete upper edge of grid
        mass=0.
        aolaptot=0.
        if(jy.eq.numygrid) then
          yolap1=latl(jy2)
          yolap2=yr
          do ii=ix1,ix2
            xolap1=max(lonl(ii),xl)
            xolap2=min(lonr(ii),xr)
            ! calculate overlapping area
            call gridarea(yolap1,abs(yolap2-yolap1),abs(xolap2-xolap1),aolap)
            mass=mass+gridin(ii,jy2,nt)*aolap
            aolaptot=aolaptot+aolap
          end do
          if(aolaptot.ne.0) then
            if(units.eq.1) then
              gridflex(ix,jy,nt)=mass/aolaptot
            else if(units.eq.2) then
              gridflex(ix,jy,nt)=mass
            else if(units.eq.3) then
              gridflex(ix,jy,nt)=mass/3600./outheight(1)/aolaptot
            endif
          endif
          outtot=outtot+mass
        endif
        ! check for undefined values
        if(isnan(gridflex(ix,jy,nt))) gridflex(ix,jy,nt)=0.
      end do ! numxgrid
    end do ! numygrid
  end do ! ntime

  ! check total in versus out
  if(check) then
    ix1=maxloc(lonl,dim=1,mask=lonr.le.outlon0)
    ix2=minloc(lonr,dim=1,mask=lonr.ge.(outlon0+(numxgrid-1)*dxout))
    jy1=maxloc(latl,dim=1,mask=latl.le.outlat0)
    jy2=minloc(latr,dim=1,mask=latr.ge.(outlat0+(numygrid-1)*dyout))
    do nt=1,ntime
      do jy=jy1,jy2
        do ix=ix1,ix2
          call gridarea(latl(jy),abs(latr(jy)-latl(jy)),abs(lonr(ix)-lonl(ix)),area)
          intot=intot+gridin(ix,jy,nt)*area
        end do
      end do
    end do
    write(logid,fmt='(1x,''Total mass in:'',1x,F10.4,1x,''Tg/y'')') intot*24.*365./float(ntime)/1.e9
    write(logid,fmt='(1x,''Total mass out:'',1x,F10.4,1x,''Tg/y'')') outtot*24.*365./float(ntime)/1.e9
    !write(logid,fmt='(1x,''Total mass in:'',1x,F10.4,1x,''Mg/y'')') intot*24.*365./float(ntime)/1.e3
    !write(logid,fmt='(1x,''Total mass out:'',1x,F10.4,1x,''Mg/y'')') outtot*24.*365./float(ntime)/1.e3
  endif

end subroutine convertgrid


