!---------------------------------------------------------------------------------------
! FLEXINVERT : errorcovariance
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculate the prior error covariance matrix using by first calculating the spatial
!! covariance at fine resolution, second averaging this to the coarse resolution and
!! lastly forming the temporal-spatial error covariance at coarse resolution.
!---------------------------------------------------------------------------------------

subroutine errorcovariance(outheight,covb,covbfin,covbfinaw)

  use mod_var

  implicit none

  character(len=100) :: varname,varunit,comment
  character(len=20) :: dname1,dname2
  logical :: lexist 
  integer :: i,j,n,ik,jk
  real :: covbsum,toterr
  real, dimension(nzmax) :: outheight
  real, dimension(:), allocatable :: xerr,xerrfin,xerrfinaw
  real, dimension(:,:), allocatable :: corspace
  real(kind=8), dimension(:,:), allocatable :: covbspace,covbtstep,lhs
  real, dimension(:,:), allocatable :: tmat,dtmat,cortime
  real, dimension(:), allocatable :: tsteps
  real, dimension(numxgrid*numygrid) :: area_vec
  real(kind=8), dimension(nemsteps*nbox+nbkgnd,nemsteps*nbox+nbkgnd) :: covb
  real(kind=8), dimension(numxnest*numynest,numxnest*numynest) :: covbfin,covbfinaw

  !---------------------------------------------------------------
  ! calculate error covariance for a single timestep
  !---------------------------------------------------------------

  allocate(corspace(numxnest*numynest,numxnest*numynest))

  ! check for existing spatial correlation
  write(nxst,'(i3.3)') numxnest
  write(nyst,'(i3.3)') numynest
  write(sigst,'(i4.4)') int(sigmaland)
  filecorrel='correl_'//trim(nxst)//'x'//trim(nyst)//'_'//trim(sigst)//'.nc'
  inquire(file=trim(dirauxil)//trim(filecorrel),exist=lexist)

  if(.not.lexist) then
    ! calculate spatial correlation
    call correl(corspace)
    varname='correl'
    varunit='none'
    dname1='dimx'
    dname2='dimy'
    comment='spatial correlation'
    call write2Dncdf(dirauxil,filecorrel,&
                     &varname,varunit,corspace,numxnest*numynest,&
                     &numxnest*numynest,dimxynest,dimxynest,dname1,dname2,comment)
  else
    ! read existing spatial correlation
    filecorrel=trim(dirauxil)//'correl_'//trim(nxst)//'x'//trim(nyst)//'_'//trim(sigst)//'.nc'
    varname='correl'
    dname1='dimx'
    dname2='dimy'
    call read2Dncdf(filecorrel,varname,dname1,dname2,&
                    &numxnest*numynest,numxnest*numynest,corspace)
  endif

  allocate(covbspace(numxnest*numynest,numxnest*numynest))
  allocate(xerr(numxnest*numynest))
  allocate(xerrfin(numxnest*numynest))
  allocate(xerrfinaw(numxnest*numynest))

  ! calculate prior error vector 
  xerrfin(:)=0.
  xerrfinaw(:)=0.
  xerr(:)=0.
  do j=1,numynest
    do i=1,numxnest
      xerr((j-1)*numxnest+i)=errprior(ixl+i-1,iyl+j-1)*area_grid(ixl+i-1,iyl+j-1)/area_box(nbox_xy(i,j))
      xerrfinaw((j-1)*numxnest+i)=errprior(ixl+i-1,iyl+j-1)*area_grid(ixl+i-1,iyl+j-1)/area_box(nbox_xy(i,j))
      xerrfin((j-1)*numxnest+i)=errprior(ixl+i-1,iyl+j-1)
    end do
  end do

  ! calculate error covariance at fine resolution
  area_vec(:)=0.
  do j=1,numynest
    do i=1,numxnest
      area_vec((j-1)*numxnest+i)=area_grid(ixl+i-1,iyl+j-1)
    end do
  end do
  covbspace(:,:)=0.
  covbsum=0.
  do j=1,numxnest*numynest
    do i=j,numxnest*numynest
      covbspace(i,j)=dble(corspace(i,j))*dble(xerr(i))*dble(xerr(j))
      covbspace(j,i)=covbspace(i,j)
      if(i.eq.j) covbsum=covbsum+covbspace(i,j)*area_vec(j)**2
      if(i.ne.j) covbsum=covbsum+2*covbspace(i,j)*area_vec(j)*area_vec(i)
    end do
  end do

  ! scale error to be consistent with total error for nested domain (in Tg/y)
  toterr=sqrt(covbsum)*outheight(1)*3600.*24.*365./numscale/1.e9
  erscalar=globerr/toterr
  covbtstep=covbtstep*erscalar**2

  write(logid,fmt='(1X,A22,F9.3)') 'Original total error: ',toterr
  write(logid,fmt='(1X,A23,F9.3)') 'Total error scaled by: ',erscalar 

  ! map prior error covariances to inversion grid

  allocate(lhs(nbox,numxnest*numynest))
  allocate(covbtstep(nbox,nbox))

  lhs=matmul(dble(gridnest),covbspace)
  deallocate(covbspace)
  covbtstep=matmul(lhs,transpose(dble(gridnest)))
  deallocate(lhs)

  ! check values in covbtstep
  do n=1,nbox
    if(covbtstep(n,n).le.0.) write(logid,fmt='(A35,1X,I6)') 'warning: covbtstep non-positive at:',n
  end do

  !---------------------------------------------------------------
  ! calculate temporal correlation over all time steps
  !---------------------------------------------------------------

  allocate(cortime(nemsteps,nemsteps))
  allocate(tsteps(nemsteps))
  allocate(tmat(nemsteps,nemsteps))
  allocate(dtmat(nemsteps,nemsteps))

  ! calculate temporal correlation as expontential decay with time (days)

  do i=1,nemsteps
    tsteps(i)=i*tstep
  end do
  tmat=matmul(reshape(tsteps,(/nemsteps,1/)),reshape((tsteps/tsteps),(/1,nemsteps/)))
  dtmat=abs(tmat-transpose(tmat))

  do i=1,nemsteps
    do j=i,nemsteps
      cortime(i,j)=exp(-dtmat(i,j)/sigmatime)
      cortime(j,i)=exp(-dtmat(j,i)/sigmatime)
    end do
  end do

  ! calculate full spatial-temporal covariance matrix as the
  ! Kroeneker product of temporal and spatial covariance matrices

  do i=1,nemsteps
    do j=i,nemsteps
      ik=(i-1)*nbox
      jk=(j-1)*nbox
      covb(ik+1:ik+nbox,jk+1:jk+nbox)=dble(cortime(i,j))*covbtstep
      covb(jk+1:jk+nbox,ik+1:ik+nbox)=dble(cortime(j,i))*covbtstep
    end do
  end do

  ! if optimize background add errors for background scalars
  ! assume the errors are not correlated in space

  if(optimizbg) then
    do i=1,nbkgnd
      covb(nbox*nemsteps+i,nbox*nemsteps+i)=0.01**2
    end do
  endif

  ! check values in covb
  do n=1,nbox*nemsteps+nbkgnd
    if(covb(n,n).le.0.) write(logid,fmt='(A30,1X,I6)') 'warning: covb non-positive at:',n
  end do

  !---------------------------------------------------------------
  ! calculate error covariance matrix on fine grid
  !---------------------------------------------------------------

  ! used in calculation of aggregation error 

  covbfin(:,:)=0.
  covbfinaw(:,:)=0.
  do j=1,numxnest*numynest
    do i=j,numxnest*numynest
      covbfin(i,j)=dble(corspace(i,j))*dble(xerrfin(i))*dble(xerrfin(j))
      covbfinaw(i,j)=dble(corspace(i,j))*dble(xerrfinaw(i))*dble(xerrfinaw(j))
      covbfin(j,i)=covbfin(i,j)
      covbfinaw(j,i)=covbfinaw(i,j)
    end do
  end do

  ! scale error to be consistent with global error
  covbfin=covbfin*(globerr/toterr)**2

  deallocate(corspace)
  deallocate(cortime)
  deallocate(tsteps)
  deallocate(tmat)
  deallocate(dtmat)
  deallocate(xerr)
  deallocate(xerrfin)
  deallocate(xerrfinaw)
  deallocate(covbtstep)

end subroutine errorcovariance





