!---------------------------------------------------------------------------------------
! FLEXINVERT : infocontent
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates the information content for the linear gaussian case in state space 
!!
!! Q = 0.5*log(det(I-A))
!!
!! where A = GH 
!! and G is the Gain matrix (H^TR^(-1)H+B^(-1))^-1
!!
!! calculate det(I-A) by first finding the QR decomposition of (I-A)
!! where Q is an orthogonal matrix and R is the upper triangular matrix
!! det(I-A) = (R11*R22*...*Rnn)**2
!!
!! Ref: Rodgers, 2000, p36 & 56
!---------------------------------------------------------------------------------------

subroutine infocontent(nn,nm,kern,gain)

  use mod_var

  implicit none

  integer :: i,nn,nm,N,M,lda,lwork,info
  real :: deta
  real(kind=8), dimension(:), allocatable :: work,tau
  real, dimension(nn,nn) :: ident
  real(kind=8), dimension(nn,nn) :: akern,iminusakern,A
  real, dimension(nm,nm) :: covr,icovr
  real(kind=8), dimension(nn,nm) :: gain 
  real, dimension(nm,nn) :: kern 

  ! calculate A = GK
  akern=matmul(gain,kern)

  ! calculate (I-A)
  ident(:,:)=0.
  do i=1,nn
    ident(i,i)=1.
  end do
  iminusakern=ident-akern

  ! QR decomposition of (I-A)
  M=nn           ! number of rows of (I-A)
  N=nn           ! number of columns of (I-A)
  A=iminusakern  ! on exit above diagonal R, below diagonal Q
  lda=nn         ! leading dimension of (I-A)
  lwork=nn       ! dimension of work
  allocate(tau(min(N,M)))
  allocate(work(lwork))
  call dgeqrf(M,N,A,lda,tau,work,lwork,info)
  if(info.ne.0) then
    write(logid,*) 'Error in QR decomposition'
    write(logid,fmt='(1X,A25,I5,A21)') 'The argument in position ',abs(info),' had an illegal value'
  endif


  ! calculate determinant of A
  deta=1.
  do i=1,M
    deta=deta*A(i,i)
  end do

  write(logid,fmt='(1X,A21,F11.3)') 'Information content: ',-0.5*log(abs(deta))

  deallocate(tau)
  deallocate(work)

end subroutine infocontent

