!---------------------------------------------------------------------------------------
! FLEXINVERT : initparam
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Initialize various global parameters
!---------------------------------------------------------------------------------------

subroutine initparam()

  use mod_var
  use mod_dates

  implicit none

  integer :: n,jy,ix
  real :: area

  !---------------------------------------------------------------
  ! initialize time parameters
  !---------------------------------------------------------------

  juldatei=juldate(datei,0)
  juldatef=juldate(datef,0)
  ngrids=int(juldatef-juldatei+1)*24*3600/abs(loutstep)
  numyr=datef/10000-datei/10000
  nummon=(datef/100-(datef/10000)*100)-(datei/100-(datei/10000)*100)+1
  numday=int(juldatef-juldatei+1)
  if(nummon.lt.0) nummon=nummon+12
  if(nummon.ge.0) nummon=nummon+numyr*12  

  !---------------------------------------------------------------
  ! initialize inversion parameters
  !---------------------------------------------------------------

  ! number of observation timesteps
  nobsteps=floor((juldatef-juldatei+1)/obstep)
  ! number of resolved emission timesteps
  nemsteps=floor((juldatef-juldatei+1)/tstep)
  ! number of releases per receptor per observation timestep
  numrel=int(obstep*24.)/relperiod

  ! list all files in observation directory and write to file 'obslist'
  call system('ls -1 -p '//trim(dirobs)//' > '//trim(dirinvert)//'obslist')
  
  !---------------------------------------------------------------
  ! initialize grid parameters
  !---------------------------------------------------------------
 
  allocate(gridlon(numxgrid))
  allocate(gridlat(numygrid))
  allocate(area_grid(numxgrid,numygrid))

  area_grid(:,:)=0.
  do jy=1,numygrid
    gridlat(jy)=outlat0+real(jy-1)*dyout
    call gridarea(gridlat(jy),dyout,dxout,area)
    do ix=1,numxgrid
      gridlon(ix)=outlon0+real(ix-1)*dxout
      area_grid(ix,jy)=area
    end do
  end do

  ! number of fine resolution gridcells in nested domain
  numxnest=int((urx-llx+1.)/dxout)
  numynest=int((ury-lly+1.)/dyout)

  allocate(nestlon(numxnest))
  allocate(nestlat(numynest))

  do jy=1,numynest
    nestlat(jy)=lly+(jy-1.)*dyout
    do ix=1,numxnest
      nestlon(ix)=llx+(ix-1.)*dxout
    end do
  end do

  ! indices on global domain for ll and ur corners of nested domain
  ixr=minloc(abs(gridlon(:)-urx),dim=1,mask=abs(gridlon(:)-urx).eq.minval(abs(gridlon(:)-urx)))
  ixl=minloc(abs(gridlon(:)-llx),dim=1,mask=abs(gridlon(:)-llx).eq.minval(abs(gridlon(:)-llx)))
  iyu=minloc(abs(gridlat(:)-ury),dim=1,mask=abs(gridlat(:)-ury).eq.minval(abs(gridlat(:)-ury)))
  iyl=minloc(abs(gridlat(:)-lly),dim=1,mask=abs(gridlat(:)-lly).eq.minval(abs(gridlat(:)-lly)))

  !---------------------------------------------------------------
  ! initialize dimension parameters
  !---------------------------------------------------------------

  allocate(dimx(numxgrid))
  allocate(dimy(numygrid))
  allocate(dimxy(numxgrid*numygrid))
  allocate(dimo(nobsteps*numrel*numrec))
  allocate(dimxnest(numxnest))
  allocate(dimynest(numynest))
  allocate(dimxynest(numxnest*numynest))

  do n=1,numxgrid
    dimx(n)=n
  end do

  do n=1,numygrid
    dimy(n)=n
  end do 

  do n=1,numxgrid*numygrid
    dimxy(n)=float(n)
  end do  

  do n=1,nobsteps*numrel*numrec
    dimo(n)=n
  enddo

  do n=1,numxnest
    dimxnest(n)=n
  end do

  do n=1,numynest
    dimynest(n)=n
  end do

  do n=1,numxnest*numynest
    dimxynest(n)=n
  end do

  !---------------------------------------------------------------
  ! write values to logfile
  !---------------------------------------------------------------

  write(logid,fmt='(1X,A19,I5)') 'Number of species: ',nspec
  write(logid,fmt='(1X,A17,I5)') 'Number of years: ',numyr
  write(logid,fmt='(1X,A18,I5)') 'Number of months: ',nummon
  write(logid,fmt='(1X,A16,I5)') 'Number of days: ',numday
  write(logid,fmt='(1X,A30,I5)') 'Number of grid files to read: ',ngrids
  write(logid,fmt='(1X,A36,I5)') 'Length of emission interval (days): ',tstep
  write(logid,fmt='(1X,A33,I5)') 'Number of observation timesteps: ',nobsteps
  write(logid,fmt='(1X,A30,I5)') 'Number of emission timesteps: ',nemsteps
  write(logid,fmt='(1X,A45,I5)') 'Number of releases per observation timestep: ',numrel
  write(logid,fmt='(1X,A33,I5)') 'Length of backtrajectory (days): ',trajlen

end subroutine initparam

