!---------------------------------------------------------------------------------------
! FLEXINVERT
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Guide to use:
!!   - assumes FLEXPART runs are separated into monthly directories
!!   - assumes a separate FLEXPART run for each receptor
!!   - example of directory structure: /STN_NAME/YYYYMM/
!!   - assumes grid_time units are s m3 kg^-1
!!   - assumes FLEXPART grid files contain uncertainties
!!     (if not need to modify readgrid and modelerror)
!!
!! Important note:
!!   - to use use bgmethod = 2 need to have one of the following set-ups
!!     1) two FLEXPART runs: i) one vertical layer for surface influence (grid_time files)  
!!        and ii) resolved vertical layers for inital conditions (grid_initial files)
!!     2) one FLEXPART run using new version: writes grid_time files with one vertical
!!        layer and grid_initial file with resolved vertical layers 
!---------------------------------------------------------------------------------------

program main

  use mod_dates
  use mod_var

  implicit none

  ! path and file variables
  character(len=100) :: filename

  ! time variables
  character(len=100) :: datestr
  integer :: yyyymm

  ! flexpart variables 
  integer :: numpoint
  real(kind=8), dimension(:), allocatable :: dateout
  real, dimension(:), allocatable :: outheight

  ! inversion variables
  integer :: n,ierr,nobs
  real :: err,cost
  real, dimension(:,:), allocatable :: surfinf,memisflex,moderr
  real, dimension(:,:,:), allocatable :: bkgnd_obs,bkgnd
  real, dimension(:), allocatable :: yobs,ymod,ybgi,yerr,ypos,errb,erra,ebgb,ebga
  real, dimension(:), allocatable :: timeout,xb,x
  real, dimension(:,:), allocatable :: knest,knest_obs,kout,kout_obs
  real, dimension(:,:), allocatable :: knest_fin,knest_finobs
  real, dimension(:,:), allocatable :: covr,cova
  real(kind=8), dimension(:,:), allocatable :: gain,covagg,covb,covbfin,covbfinaw
  integer, dimension(:), allocatable :: ydat
  character(len=7), dimension(:), allocatable :: yrec
  real(kind=8), dimension(:,:,:), allocatable :: allobs,modout  


  !---------------------------------------------------------------
  ! read inversion definitions
  !---------------------------------------------------------------
 
  call readdefinitions()

  call system('cp control.def  '//trim(dirinvert))
  call system('cp files.def  '//trim(dirinvert))

  !---------------------------------------------------------------
  ! read list of receptors/stations
  !---------------------------------------------------------------

  call readreclist()

  !---------------------------------------------------------------
  ! read flexpart header
  !---------------------------------------------------------------

  ! read header for only one receptor as this must be the same for
  ! all receptors/stations

  yyyymm=datei/100
  write(datestr,'(i6)') yyyymm
  if(usedirflex) then
    filename=trim(dirflex)//trim(recname(1))//'/'//trim(datestr)//'/header_grid_time'
  else
    filename=trim(dirflex)//trim(recname(1))//'/'//trim(datestr)//'/header'
  endif

  allocate(outheight(nzmax))
  outheight(:)=0.

  call readheader(filename,numpoint,outheight)

  !---------------------------------------------------------------
  ! initialize various parameters 
  !---------------------------------------------------------------

  call initparam()  

  !---------------------------------------------------------------
  ! calculate mean surface influence
  !---------------------------------------------------------------
  
  ! read the individual gridtime files at each timestep and sum
  ! the surface influence over each receptor and the inversion
  ! time interval

  allocate(surfinf(numxgrid,numygrid))

  call calcmeansurfinf(surfinf)

  !---------------------------------------------------------------
  ! read prior fluxes
  !---------------------------------------------------------------

  ! read prior fluxes from ncdf file

  call reademissions()

  allocate(emisflex(numxgrid,numygrid,ntemi))
  allocate(memisflex(numxgrid,numygrid))

  ! map fluxes to flexpart grid (for use in mapgrid)
  ! second argument is unit of output (assumes input unit is kg/m2/h):
  ! unit = 1: flux (kg/m2/h)
  ! unit = 2: mass (kg/h)
  ! unit = 3: volume mixing ratio (kg/m3/s)

  call convertgrid(.true.,3,outheight,nxemi,nyemi,lonemi,latemi,ntemi,emis,emisflex)

  ! annual mean emission for use in mapgrid

  memisflex=sum(emisflex,dim=3)

  !---------------------------------------------------------------
  ! calculate inversion grid
  !---------------------------------------------------------------

  ! calculate the grid for inversion by aggregating grid cells of 
  ! low surface influence together

  ! alternate inversion grid definitions can be used for
  ! nbox_xy(numxnest,numynest) in which case replace
  ! routine mapgrid with a new grid definition

  allocate(xlanduse(numxnest,numynest,nclass))

  call readlanduse()

  ! create grid for inversion (for nested domain)

  allocate(nbox_xy(numxnest,numynest))

  call mapgrid(surfinf(ixl:ixr,iyl:iyu),memisflex(ixl:ixr,iyl:iyu))
                
  ! read land-sea mask

  call readlsm()

  ! convert land-sea mask to flexpart grid

  allocate(lsmflex(numxgrid,numygrid,1))

  call convertgrid(.false.,1,outheight,nxlsm,nylsm,lonlsm,latlsm,1,lsm,lsmflex)

  ! apply land-sea mask to split boxes into land and sea boxes
  ! to reduce the spatial aggregation error (not yet implemented!)

  ! calculate area of boxes on inversion grid

  call boxarea() 

  ! calculate grid interpolation operator based on nbox_xy

  call gridoperator()

  ! calculate grid interpolation operator for coarse grid used
  ! for calculating the background

  call gridcoarse()

  if(optimizbg) then
    if(bgmethod.eq.1) nbkgnd=nlatbin
    if(bgmethod.eq.2) nbkgnd=ncrs
  else
    nbkgnd=0
  endif

  !---------------------------------------------------------------
  ! calculate the source-receptor matrix (kernel)
  !---------------------------------------------------------------

  ! reads surface influences for each timestep and receptor and
  ! writes these to the source-receptor matrix for the nested 
  ! domain (knest) and for outside the nested domain (kout)

  allocate(knest(numrec*(numday*24/relperiod),nemsteps*nbox))
  allocate(knest_fin(numrec*(numday*24/relperiod),numxnest*numynest))
  allocate(kout(numrec*(numday*24/relperiod),ncrs))
  allocate(moderr(numday*24/relperiod,numrec))
  knest(:,:)=0.
  knest_fin(:,:)=0.
  kout(:,:)=0.
  moderr(:,:)=0.

  call calcsrmatrix(knest,knest_fin,kout,moderr)

  !---------------------------------------------------------------
  ! read observations and format knest and kout to match
  !---------------------------------------------------------------

  allocate(allobs(nobsteps,numrec,6))
  allocate(knest_obs(nobsteps*numrec,nemsteps*nbox))
  allocate(knest_finobs(nobsteps*numrec,numxnest*numynest))
  allocate(kout_obs(nobsteps*numrec,ncrs))
  allobs(:,:,:)=0.
  knest_obs(:,:)=0.
  knest_finobs(:,:)=0.
  kout_obs(:,:)=0.

  call calcobsvector(knest,knest_fin,kout,knest_obs,knest_finobs,kout_obs,allobs,moderr)

  !---------------------------------------------------------------
  ! scale knest, knest_fin and fluxes for better numerical handling
  !---------------------------------------------------------------

  knest=knest/numscale
  knest_obs=knest_obs/numscale
  knest_finobs=knest_finobs/numscale
  knest_fin=knest_fin/numscale
  emisflex=emisflex*numscale
  maxerr=maxerr*numscale
  minerr=minerr*numscale

  !---------------------------------------------------------------
  ! calculate the state vector 
  !---------------------------------------------------------------

  ! map fluxes to inversion grid and vectorize

  allocate(emisprior(numxgrid,numygrid,nemsteps))
  allocate(xb(nbox*nemsteps+nbkgnd))
  allocate(timeout(nemsteps))
  xb(:)=0.

  call calcstatevector(xb,timeout)

  !---------------------------------------------------------------
  ! calculate background mixing ratios
  !---------------------------------------------------------------

  allocate(bkgnd_obs(nobsteps,numrec,ncrs))
  allocate(bkgnd(numday*24/relperiod,numrec,ncrs))
  bkgnd(:,:,:)=0.
  bkgnd_obs(:,:,:)=0.

  ! calculate background at each timestep from observations
  if(bgmethod.eq.1) then
    call backgroundobs(allobs,knest_obs,xb)
  endif

  ! calculate background using initial conditions and sensitivity
  if(bgmethod.eq.2) then
    call backgroundini(allobs,bkgnd_obs,bkgnd)
  endif

  !---------------------------------------------------------------
  ! calculate prior mixing ratios and observation error
  !---------------------------------------------------------------

  allocate(yobs(nobsteps*numrec))
  allocate(ymod(nobsteps*numrec))
  allocate(ybgi(nobsteps*numrec))
  allocate(yerr(nobsteps*numrec))
  allocate(ydat(nobsteps*numrec))
  allocate(yrec(nobsteps*numrec))

  ! reuse variable names for final thinned kernel  
  deallocate(knest)
  deallocate(knest_fin)
  allocate(knest(nobsteps*numrec,nemsteps*nbox+nbkgnd))
  allocate(knest_fin(nobsteps*numrec,numxnest*numynest))

  call calcprior(knest_obs,knest,knest_finobs,knest_fin,kout_obs,&
                 &bkgnd_obs,xb,allobs,yobs,ymod,ybgi,yerr,ydat,yrec,nobs)

  ! if checkprior = true: save output and exit
  if(checkprior) go to 10

  !---------------------------------------------------------------
  ! calculate prior flux error
  !---------------------------------------------------------------

  call calcpriorerror(outheight)

  !---------------------------------------------------------------
  ! calculate prior error covariance matrix
  !---------------------------------------------------------------

  allocate(covb(nemsteps*nbox+nbkgnd,nemsteps*nbox+nbkgnd))
  allocate(covbfin(numxnest*numynest,numxnest*numynest))
  allocate(covbfinaw(numxnest*numynest,numxnest*numynest))

  ! select one routine for calculation on fine grid and map to 
  ! variable grid (slow) or directly on variable grid (less slow)
!  call errorcovariance(outheight,covb,covbfin,covbfinaw)
  call errorcovariance_coarse(outheight,covb,covbfin,covbfinaw)

  !---------------------------------------------------------------
  ! calculate observation error covariance matrix
  !---------------------------------------------------------------

  allocate(covr(nobs,nobs))

  call obscovariance(covr,yerr,nobs)

  !---------------------------------------------------------------
  ! calculate spatial aggregation error
  !---------------------------------------------------------------

  allocate(covagg(nobs,nobs))

  call erroraggreg(numxnest*numynest,nobs,nbox,covbfinaw,knest_fin(1:nobs,:),covagg)  

  ! add aggregation error to observation error
  covr=covr+real(covagg,kind=4) 

  !---------------------------------------------------------------
  ! add random perturbation to xb and yobs
  !---------------------------------------------------------------

  ! in this case xb are true emissions and ymod the observations
  ! xb are perturbed to form the prior and yobs are perturbed to introduce noise

  if(perturb) then
    ! returns xb and yobs with perturbations
    call calcperturb(nbox*nemsteps+nbkgnd,nobs,covb,covr,xb,yobs(1:nobs),ymod(1:nobs),outheight)
  endif

  !---------------------------------------------------------------
  ! calculate cost a priori
  !---------------------------------------------------------------
  
  call calccost(1,nbox*nemsteps+nbkgnd,nobs,xb,xb,yobs(1:nobs),knest(1:nobs,:),covr,covb,cost)

  !---------------------------------------------------------------
  ! inversion calculations
  !---------------------------------------------------------------

  allocate(gain(nbox*nemsteps+nbkgnd,nobs))
  allocate(cova(nemsteps*nbox+nbkgnd,nemsteps*nbox+nbkgnd))
  allocate(x(nbox*nemsteps+nbkgnd))

  ! fewer observations than state variables

  if(nobs.le.nemsteps*nbox+nbkgnd) then
    call bayesian1(nbox*nemsteps+nbkgnd,nobs,covb,covr,yobs(1:nobs),knest(1:nobs,:),xb,x,cova,gain)
  endif

  ! fewer state variables than observations

  if(nobs.gt.nemsteps*nbox+nbkgnd) then
     call bayesian2(nbox*nemsteps+nbkgnd,nobs,covb,covr,yobs(1:nobs),knest(1:nobs,:),xb,x,cova,gain)
  endif

  !---------------------------------------------------------------
  ! information content
  !---------------------------------------------------------------

  call infocontent(nbox*nemsteps+nbkgnd,nobs,knest(1:nobs,:),gain)

  !---------------------------------------------------------------
  ! calculate cost a posteriori
  !---------------------------------------------------------------

  call calccost(2,nbox*nemsteps+nbkgnd,nobs,x,xb,yobs(1:nobs),knest(1:nobs,:),covr,covb,cost)

  !---------------------------------------------------------------
  ! map fluxes and errors back to flexpart grid
  !---------------------------------------------------------------

  allocate(errb(nbox*nemsteps))
  allocate(erra(nbox*nemsteps))
  allocate(emispost(numxnest,numynest,nemsteps))
  allocate(errpost(numxnest,numynest,nemsteps))
  allocate(errpri(numxnest,numynest,nemsteps))

  ! prior and posterior flux errors
!  do n=1,nbox*nemsteps
!    errb(n)=real(sqrt(covb(n,n)),kind=4)
!    erra(n)=real(sqrt(cova(n,n)),kind=4)
!  end do
  do n=1,nbox*nemsteps
    errb(n)=real(sqrt(sum(abs(covb(:,n)))),kind=4)
    erra(n)=real(sqrt(sum(abs(cova(:,n)))),kind=4)
  end do

  ! if optimize background: prior and posterior background scalar errors
  if(optimizbg) then
    allocate(ebgb(nbkgnd))
    allocate(ebga(nbkgnd))
    do n=1,nbkgnd
      ebgb(n)=real(sqrt(covb(nbox*nemsteps+n,nbox*nemsteps+n)),kind=4)
      ebga(n)=real(sqrt(cova(nbox*nemsteps+n,nbox*nemsteps+n)),kind=4)
    end do
  endif

  ! re-grid posterior emissions on fine grid
  ! if scaleflx = true : uses spatial distribution of prior fluxes

  call regridemissions(x(1:nbox*nemsteps),xb(1:nbox*nemsteps),emispost)

  ! do the same for the posterior errors

  scaleflx=.false.
  call regridemissions(erra,xb(1:nbox*nemsteps),errpost)
  
  ! and for the prior errors

  call regridemissions(errb,xb(1:nbox*nemsteps),errpri)

  ! alternatively use Bayesian optimization of fluxes to fine grid
  ! (note that this is still experimental)  
  
!  call regridoptimize(numxnest*numynest,nbox,covbfin,covbfinaw,covb,cova,xb,x)

  !---------------------------------------------------------------
  ! calculate prior and posterior concentrations
  !---------------------------------------------------------------

  ! convert emissions back from volume mixing ratio to flux units (kg/m2/s)
  emisflex=emisflex*outheight(1)/numscale
  emisprior=emisprior*outheight(1)/numscale
  ! scalar prior error for consistency with covb (see errorcovariance.f90)
!  errprior=errprior*outheight(1)*erscalar/numscale
  emispost=emispost*outheight(1)/numscale
  errpost=errpost*outheight(1)/numscale
  errpri=errpri*outheight(1)/numscale

  ! read gridded flexpart output files to calculate surface influence
  ! and multiply with the prior and posterior fluxes

  allocate(modout(numday*24/relperiod,numrec,5))
  allocate(dateout(numday*24/relperiod))

  write(logid,*) 'Calculating posterior model mixing ratios at full resolution...'

  yyyymm=datei/100
  write(datestr,'(i6)') yyyymm
  if(usedirflex) then
    filename=trim(dirflex)//trim(recname(1))//'/'//trim(datestr)//'/header_grid_time'
  else
    filename=trim(dirflex)//trim(recname(1))//'/'//trim(datestr)//'/header'
  endif

  call readheader(filename,numpoint,outheight)

  call timeseries(bkgnd,kout,outheight,timeout,modout,dateout,allobs,xb,x)

  !---------------------------------------------------------------
  ! save various things to files
  !---------------------------------------------------------------

  call save(outheight,nobs,knest,knest_obs,kout_obs,knest_finobs,&
            &covb,covbfin,covbfinaw,covr,covagg,cova,gain,xb,x,yrec,&
            &ydat,yobs,ymod,ybgi,yerr,ebgb,ebga,modout,dateout,timeout)

10 continue

  if(checkprior) then
    call save_forward(nobs,yrec,ydat,yobs,ymod,ybgi,yerr)
  endif

  !---------------------------------------------------------------
  ! prepare to exit
  !---------------------------------------------------------------

  deallocate(latbin)
  deallocate(outheight)
  deallocate(surfinf)
  deallocate(lonemi)
  deallocate(latemi)
  deallocate(timemi)
  deallocate(emis)
  deallocate(emisflex)
  deallocate(memisflex)
  deallocate(xlanduse)
  deallocate(nbox_xy)
  deallocate(lonlsm)
  deallocate(latlsm)
  deallocate(lsm)
  deallocate(lsmflex)
  deallocate(gridoper)
  deallocate(knest)
  deallocate(knest_fin)
  deallocate(kout)
  deallocate(moderr)
  deallocate(allobs)
  deallocate(knest_obs)
  deallocate(knest_finobs)
  deallocate(bkgnd)
  deallocate(bkgnd_obs)
  deallocate(emisprior)
  deallocate(xb)
  deallocate(timeout)
  deallocate(yobs)
  deallocate(ymod)
  deallocate(ybgi)
  deallocate(yerr)
  deallocate(ydat)
  deallocate(yrec)

  if(checkprior) go to 20 

  deallocate(covb)
  deallocate(covbfin)
  deallocate(covbfinaw)
  deallocate(covr)
  deallocate(covagg)
  deallocate(gain)
  deallocate(cova)
  deallocate(x)
  deallocate(errb)
  deallocate(erra)
  deallocate(emispost)
  deallocate(errpost)
  deallocate(errprior)
  deallocate(modout)
  deallocate(dateout)
  deallocate(area_grid)
  deallocate(gridlon)
  deallocate(gridlat)
  deallocate(nestlon)
  deallocate(nestlat)
  deallocate(area_box)
  deallocate(dimx)
  deallocate(dimy)
  deallocate(dimxnest)
  deallocate(dimynest)
  deallocate(dimxynest)
  deallocate(dimxy)
  deallocate(dimr)
  deallocate(dimi)
  deallocate(dimo)
  deallocate(dimbox)
  deallocate(dimb)
  deallocate(dimbx)
  deallocate(recname)
  deallocate(rectyp)  
  deallocate(reclon)
  deallocate(reclat)
  deallocate(recalt)
 
20 continue

  write(logid,*) 'Good-bye'
  close(logid)

end program main

