!---------------------------------------------------------------------------------------
! FLEXINVERT : mapgrid
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Based on mapgrid.f routine of A. Stohl
!!
!! This subroutine lumps grid cells with low emission 
!! sensitivity into larger boxes. This is done in nmap steps, 
!! starting with the coarsest boxes (resolution nres set by 
!! mapdec). When both the sensitivity is among the "cutoff" 
!! lowest values, boxes are used at coarse resolution. If they 
!! have higher sensitivity values, they are taken to the next
!! step (i.e., boxes are split to higher resolution). For boxes
!! whose total sensitivity is below sensitivity_min, boxes are
!! kept but not actually used for matrix inversion
!!
!! options: 
!!   useemission : (true) weight surface influnce by annual emissions
!!   uselanduse: (true) avoid gridcells with > 99% water
!---------------------------------------------------------------------------------------

subroutine mapgrid(surfinf,emission)

  use mod_var

  implicit none

  real, dimension(numxnest,numynest) :: surfinf,emission
  integer ::  n,i,ix,jy,iix,jjy,i1,j1,ii,jj,nres,ndec,nfx,nfy,nfbox,nsort
  real :: emis_average
  integer, parameter :: nmap=3,maxbox=10000
  real, parameter :: sensitivity_min=1.e13,cutoff=0.8
  integer, dimension(nmap) :: mapdec(nmap)
  integer, dimension(2*maxbox) :: map_box
  integer :: nbox_invert,nboxnotinvert,nb
  real, dimension(2*maxbox) :: x_map,y_map,sens_map,water_box
  real, dimension(numxnest,numynest) :: sens_lump,contrib_lump,water
  real, dimension(numxnest*numynest) :: sort_contrib,sort_contrib_inv
  logical, dimension(numxnest,numynest) :: mark
  real :: cut_contrib

  ! specify resolution steps (number of steps must equal nmap!)
  mapdec = (/1,2,2/)

  ! initialize some variables
  nbox_invert=0
  nboxnotinvert=maxbox
  nbox_xy(:,:)=0
  sens_lump(:,:)=0.
  contrib_lump(:,:)=0.
  sort_contrib(:)=0.
  ! mean emission over land
  emis_average=sum(emission)/float(numxnest*numynest)

  ! increase resolution stepwise from coarsest to finest grid
  ! first iteration at resolution equal to product of mapdec
  do ndec=nmap,2,-1
    nres=1
    do i=1,ndec
      nres=nres*mapdec(i)
    end do

    ! lump data on coarse grid
    nfx=0
    do ix=1,numxnest-nres+2,nres
      nfx=nfx+1
      nfy=0
      do jy=1,numynest-nres+1,nres
        nfy=nfy+1
        sens_lump(nfx,nfy)=0.
        contrib_lump(nfx,nfy)=0.
        water(nfx,nfy)=0.
        mark(nfx,nfy)=.false.
        ! sens_lump : contains sum of surface influences from all grid cells at 
        ! fine resolution corresponding to the grid cell at coarse resolution 
        do i1=0,nres-1
          do j1=0,nres-1
            if ((ix+i1).gt.numxnest) then 
              ii=i1-1 
            else 
              ii=i1
            end if
            jj=j1
            if (nbox_xy(ix+ii,jy+jj).gt.0) then
              mark(nfx,nfy)=.true.
            else
              sens_lump(nfx,nfy)=sens_lump(nfx,nfy)+surfinf(ix+ii,jy+jj)
              if(useemission) then
                ! contrib_lump : surface influence times emission flux
                ! take a certain minimum emission flux for sorting
                contrib_lump(nfx,nfy)=contrib_lump(nfx,nfy)+surfinf(ix+ii,jy+jj)*emission(ix+ii,jy+jj)
!                contrib_lump(nfx,nfy)=contrib_lump(nfx,nfy)+surfinf(ix+ii,jy+jj)*&
!                       &max(emis_average/10.,emission(ix+ii,jy+jj))
              else
                ! contrib_lump : surface influence only
                contrib_lump(nfx,nfy)=contrib_lump(nfx,nfy)+surfinf(ix+ii,jy+jj)
              endif
              if(uselanduse) then
                ! fraction of water (7) and ice (12)
                water(nfx,nfy)=water(nfx,nfy)+xlanduse(ix+ii,jy+jj,7)+xlanduse(ix+ii,jy+jj,12)
              else
                ! set to zero to avoid landuse consideration
                water(nfx,nfy)=0.
              endif
            endif
          end do ! nres
        end do ! nres
        water(nfx,nfy)=water(nfx,nfy)/float(nres)**2
      end do ! numynest
    end do ! numxnest
    nfbox=nfx*nfy

    ! sort values in decreasing order and determine cut-off 
    nsort=0
    do iix=1,nfx
      do jjy=1,nfy
        if (.not.mark(iix,jjy)) then
          nsort=nsort+1
          sort_contrib(nsort)=contrib_lump(iix,jjy)
        endif
      end do ! nfy
    end do ! nfx
    ! sort: orders in increasing order
    call sort(nsort,sort_contrib(1:nsort))
    ! change to decreasing order
    sort_contrib_inv(:)=0.
    do n=1,nsort
      sort_contrib_inv(nsort-n+1)=sort_contrib(n)
    end do
    sort_contrib=sort_contrib_inv
    cut_contrib=sort_contrib(nint(cutoff*float(nsort))+1)

    ! for all lumped boxes below threshold, mark lumped box, which will
    ! subsequently not be subdivided any further
    do iix=1,nfx
      do jjy=1,nfy
        ! changed following line: .lt. to .le. for instances when cut_contrib=0
        if ((.not.mark(iix,jjy)).and.((contrib_lump(iix,jjy).le.cut_contrib).or.(water(iix,jjy).gt.0.99))) then
          ! if box all over water (and uselanduse is .true.) do not use box
          ! if sensitivity in box is too small, also do not use it
          if ((water(iix,jjy).gt.0.99).or.(sens_lump(iix,jjy).lt.sensitivity_min)) then 
            ! do not use box for matrix inversion
            nboxnotinvert=nboxnotinvert+1
            if (nboxnotinvert.gt.2*maxbox) then
              write(logid,*) 'Error: maxbox too small'
              stop
            endif
            nb=nboxnotinvert
          else                                            
            ! use box for matrix inversion
            nbox_invert=nbox_invert+1
            if (nbox_invert.gt.maxbox) then
              write(logid,*) 'Error: maxbox too small'
              stop
            endif
            nb=nbox_invert
          endif
          map_box(nb)=ndec
          x_map(nb)=iix
          y_map(nb)=jjy
          sens_map(nb)=sens_lump(iix,jjy)
          water_box(nb)=water(iix,jjy)
          do ix=(iix-1)*nres+1,iix*nres
            do jy=(jjy-1)*nres+1,jjy*nres
              ii=min(numxnest,ix)
              jj=min(numynest,jy)
              nbox_xy(ii,jj)=nb
            end do
          end do
        endif
      end do ! nfy
    end do ! nfx

  end do ! loop over nmap

  ! for all remaining boxes, use the full-resolution model grid
  ndec=1
  do ix=1,numxnest
    do jy=1,numynest
      if (nbox_xy(ix,jy).eq.0) then
        ! if box all over water and landuse option is set, do not use box
        ! if sensitivity in box is too small, also do not use it
        if ((xlanduse(ix,jy,7)+xlanduse(ix,jy,12).gt.0.99).or.(surfinf(ix,jy).lt.sensitivity_min)) then 
          ! do not use box for matrix inversion
          nboxnotinvert=nboxnotinvert+1
          if (nboxnotinvert.gt.2*maxbox) then
            write(logid,*) 'Error: maxbox too small'
            stop
          endif    
          nb=nboxnotinvert
        else                                                
          ! use box for matrix inversion
          nbox_invert=nbox_invert+1
          if (nbox_invert.gt.maxbox) then
            write(logid,*) 'Error: maxbox too small'
            stop
          endif    
          nb=nbox_invert
        endif
        map_box(nb)=ndec
        x_map(nb)=ix
        y_map(nb)=jy
        sens_map(nb)=surfinf(ix,jy)
        water_box(nb)=water(ix,jy)
        nbox_xy(ix,jy)=nb
      endif
    end do ! numxnest
  end do ! numynest

  ! now arrange the inversion and not-inversion boxes such that inversion boxes come first in array,
  ! and not-inversion boxes come immediately afterwards

  nbox=nbox_invert
  do nb=maxbox+1,nboxnotinvert
    nbox=nbox+1
    if (nbox.gt.maxbox) then
      write(logid,*) 'Error: maxbox too small'
      stop
    endif
    map_box(nbox)=map_box(nb)
    x_map(nbox)=x_map(nb)
    y_map(nbox)=y_map(nb)
    sens_map(nbox)=sens_map(nb)
    water_box(nbox)=water_box(nb)
    do ix=1,numxnest
      do jy=1,numynest
        if (nbox_xy(ix,jy).eq.nb) nbox_xy(ix,jy)=nbox
      end do
    end do
  end do

end subroutine mapgrid
