!---------------------------------------------------------------------------------------
! FLEXINVERT : readlanduse
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Based on script readlanduse.f by A. Stohl
!!
!! Modified by  S.Eckhardt for new landuse inventory (Dec 06)    
!! Ref:  Belward, A.S., Estes, J.E., and Kline, K.D., 1999,                                    
!! The IGBP-DIS 1-Km Land-Cover Data Set DISCover:                               
!! A Project Overview: Photogrammetric Engineering and Remote Sensing,        
!! v. 65, no. 9, p. 1013-1020
!!
!! Reads the landuse inventory into memory and relates it to 
!! Leaf Area Index and roughness length.
!!
!! LANDUSE CATEGORIES:                                                       
!!                                                                            
!! 1	Urban land                                                             	
!! 2	Agricultural land					                       
!! 3	Range land					                       
!! 4	Deciduous forest			                               
!! 5	Coniferous forest		                                       
!! 6	Mixed forest including wetland                                         
!! 7	water, both salt and fresh                                             
!! 8	barren land mostly desert                                                      
!! 9	nonforested wetland	                                                
!! 10 	mixed agricultural and range land		                       
!! 11 	rocky open areas with low growing shrubs                               
!! 12   ice                                                                    
!! 13   rainforest      
!---------------------------------------------------------------------------------------

subroutine readlanduse()

  use mod_var

  implicit none

  integer :: ix,jy,k,lu_cat,lu_perc,li,ierr
  integer :: nrefine,iix,jjy,ixi,jyj
  integer(kind=1) :: ilr
  integer, dimension(1200,600,6) :: landinvent
  integer :: il,irecread
  real :: rlr,r2lr
  real, dimension(1200,600,nclass) :: xlandusep
  real, parameter :: xlon0lu=-180.,ylat0lu=-90.
  real :: xlon,ylat,sumperc,p,pn,perck

  xlanduse(:,:,:)=0.

  !--------------------------------------------------------------
  ! Read landuse inventory
  !--------------------------------------------------------------
   
  ! The landuse information is saved in a compressed format and written
  ! out by records of the length of 1 BYTE. Each grid cell consists of 3
  ! Bytes, which include 3 landuse categories (val 1-13 and 16 percentage
  ! categories) So one half byte is used to store the Landusecat the other
  ! for the percentageclass in 6.25 steps (100/6.25=16)
  ! Example: 
  ! 4 3  percentage 4 = 4*6.25 => 25% landuse class 3
  ! 2 1  percentage 2 = 2*6.25 => 13% landuse class 1
  ! 1 12 percentage 1 = 1*6.25 => 6.25% landuse class 12     

  open(100,file=trim(filelanduse),status='old',access='DIRECT',form='UNFORMATTED',iostat=ierr,recl=1)
  if(ierr.gt.0) write(logid,*) 'Error reading landuse file: '//trim(filelanduse)

  irecread=1
  do ix=1,1200
    do jy=1,600
      ! the 3 most abundant landuse categories in the inventory        
      ! first half byte contains the landuse class
      ! second half byte contains the respective percentage        
      do k=1,3
        ! 1 byte is read
        read(100,rec=irecread) ilr
        irecread=irecread+1
        ! as only signed integer values exist an unsigned value is constructed
        if (ilr.lt.0) then 
          il=ilr+256
        else 
          il=ilr
        endif
        ! dividing by 16 has the effect to get rid of the right half of the byte            
        ! so just the left half remains, this corresponds to a shift right of 4
        ! bits            
        rlr=real(il)/16.
        lu_cat=int(rlr)
        ! the left half of the byte is substracted from the whole in order to 
        ! get only the right half of the byte            
        r2lr=rlr-int(rlr)
        ! shift left by 4
        lu_perc=r2lr*16.
        landinvent(ix,jy,k)=lu_cat
        landinvent(ix,jy,k+3)=lu_perc
      end do ! k
    end do ! jy
  end do ! ix

  close(100) 

  !--------------------------------------------------------------
  ! Assign the landuse to our grid
  !--------------------------------------------------------------

  do ix=1,1200  
    do jy=1,600
      do k=1,nclass
        xlandusep(ix,jy,k)=0.
      end do
      sumperc=0.
      do li=1,3
        sumperc=sumperc+landinvent(ix,jy,li+3)
      end do
      do li=1,3
        k=landinvent(ix,jy,li)
        if (sumperc.gt.0) then
          p=landinvent(ix,jy,li+3)/sumperc
        else
          p=0
        endif
        ! p has values between 0 and 1             
        xlandusep(ix,jy,k)=p
!        xlandusep(ix,jy,k)=landinvent(ix,jy,li+3)/100.
      end do ! li
    end do ! jy
  end do ! ix

  nrefine=10
  do ix=1,numxnest
    do jy=1,numynest
      do k=1,nclass
        xlanduse(ix,jy,k)=0.
      end do ! nclass
      pn=0.
      do iix=1,nrefine
        xlon=(ix-1+(iix-1)/float(nrefine))*dxout+llx        
        do jjy=1,nrefine
          ylat=(jy-1+(jjy-1)/float(nrefine))*dyout+lly 
          ixi=int((xlon-xlon0lu)/0.3)+1
          jyj=int((ylat-ylat0lu)/0.3)+1
          if (ixi.gt.1200) ixi=ixi-1200
          if (jyj.gt.600) jyj=600
          perck=0.
          do k=1,nclass
            xlanduse(ix,jy,k)=xlanduse(ix,jy,k)+xlandusep(ixi,jyj,k)
            perck=perck+xlandusep(ixi,jyj,k)
          end do ! nclass
!          if (perck.gt.(1-1E-5)) pn=pn+1
        end do
        do k=1,nclass
!          xlanduse(ix,jy,k)=xlanduse(ix,jy,k)/pn
          xlanduse(ix,jy,k)=xlanduse(ix,jy,k)/float(nrefine)
        end do 
      end do ! nrefine
    end do ! numynest
  end do ! numxnest

end subroutine readlanduse
