!---------------------------------------------------------------------------------------
! FLEXINVERT : readlmdz 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads mixing ratios from global fields from LMDZ model runs (LMDZ96 grid) 
!! note:
!!  - particular set-up is for LMDZ96 output files (runch4_online_test5)
!!  - time in LMDZ96 file is seconds since 1979-01-01 00:00:00
!!  - pressure in LMDZ96 is Pa
!!  - variable in LMDZ96 is volume mixing ratio
!---------------------------------------------------------------------------------------

subroutine readlmdz(jd)

  use mod_var
  use mod_dates

  implicit none

  include 'netcdf.inc'

  character(len=100) :: filename,filepart1,filepart2,filepart3
  character(len=4) :: yyyystr
  character(len=2) :: mmstr
  integer :: iy,i,nti,it,iz,ind1,ind2,cnt,nt
  integer :: yyyymmddhh,yyyymmdd,yyyymm,mm,yyyy
  integer :: nid,yid,xid,zid,tid,ierr,lonid,latid,prsid,timid,varid
  real(kind=8) :: jd
  real, dimension(:), allocatable :: lato
  integer, dimension(:), allocatable :: indy
  real, dimension(:,:), allocatable :: tmp
  real, dimension(:), allocatable :: tmptime
  real, dimension(:,:,:,:), allocatable :: tmpt,tmpimmr


  !--------------------------------------------------------------
  ! read file from previous month
  !--------------------------------------------------------------

  if((jd-trajlen).ge.juldatei) then

    ! define exact filename
    yyyymmddhh=caldate(jd-trajlen)
    yyyymmdd=yyyymmddhh/100
    yyyy=yyyymmddhh/1000000
    mm=(yyyymmddhh-yyyy*1000000)/10000
    ind1=index(fileimmr,'YYYY')-1
    filepart1=fileimmr(1:ind1)
    ind2=index(fileimmr,'MM')-1
    filepart2=fileimmr((ind1+5):ind2)
    filepart3=fileimmr((ind2+3):len(fileimmr))
    write(yyyystr,'(i4)') yyyy
    write(mmstr,'(i2.2)') mm
    filename=trim(filepart1)//yyyystr//trim(filepart2)//mmstr//trim(filepart3)

    ! open netcdf file
    write(logid,*) 'Reading the file: '//trim(filename)
    ierr=nf_open(trim(filename),NF_NOWRITE,nid)
    if(ierr.gt.0) then
      write(logid,*) 'Error opening file: '//trim(filename)
    endif

    ! inquire about variables
    ierr=nf_inq_dimid(nid,'x',xid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: x'
    endif
    ierr=nf_inq_dimid(nid,'y',yid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: y'
    endif
    ierr=nf_inq_dimid(nid,'presnivs',zid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: presnivs'
    endif
    ierr=nf_inq_dimid(nid,'time_counter',tid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: time_counter'
    endif

    ! read dimension sizes
    ierr=nf_inq_dimlen(nid,xid,nxini)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif
    ierr=nf_inq_dimlen(nid,yid,nyini)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif
    ierr=nf_inq_dimlen(nid,zid,nzini)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif
    ierr=nf_inq_dimlen(nid,tid,nt)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif

    ! allocate variables
    allocate(lonini(nxini))
    allocate(latini(nyini))
    allocate(lato(nyini))
    allocate(indy(nyini))
    allocate(pres(nzini))
    allocate(tmptime(nt))
    allocate(temp(nzini))
    allocate(tmp(nxini,nyini))
    allocate(tmpt(nxini,nyini,nzini,nt))
    allocate(tmpimmr(nxini,nyini,nzini,nt))

    ! allocate immrraw and timini with max time dimension
    allocate(immrraw(nxini,nyini,nzini,30))
    allocate(timini(30))

    ! read variables
    ierr=nf_inq_varid(nid,'nav_lon',lonid)
    ierr=nf_get_var_real(nid,lonid,tmp)
    lonini=tmp(:,1)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: nav_lon'
    endif
    ierr=nf_inq_varid(nid,'nav_lat',latid)
    ierr=nf_get_var_real(nid,latid,tmp)
    latini=tmp(1,:)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: nav_lat'
    endif
    ierr=nf_inq_varid(nid,'presnivs',prsid)
    ierr=nf_get_var_real(nid,prsid,pres)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: presnivs'
    endif
    ierr=nf_inq_varid(nid,'time_counter',timid)
    ierr=nf_get_var_real(nid,timid,tmptime)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: time_counter'
    endif
    ierr=nf_inq_varid(nid,trim(bginame),varid)
    ierr=nf_get_var_real(nid,varid,tmpimmr)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: ',trim(bginame)
    endif
    ierr=nf_inq_varid(nid,'temp',varid)
    ierr=nf_get_var_real(nid,varid,tmpt)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: temp'
    endif

    ! close file
    ierr=nf_close(nid)

    ! assign to immrraw and timini
    immrraw(:,:,:,1:nt)=tmpimmr
    timini(1:nt)=tmptime

    cnt=nt+1

    ! deallocate variables with time dimension to prepare for current month
    deallocate(tmptime)
    deallocate(tmpt)
    deallocate(tmpimmr)

  endif ! if((jd-trajlen).ge.juldatei)

  !--------------------------------------------------------------
  ! read file from current month
  !--------------------------------------------------------------

  ! define exact filename
  yyyymmddhh=caldate(jd)
  yyyymmdd=yyyymmddhh/100
  yyyy=yyyymmddhh/1000000
  mm=(yyyymmddhh-yyyy*1000000)/10000
  ind1=index(fileimmr,'YYYY')-1
  filepart1=fileimmr(1:ind1)
  ind2=index(fileimmr,'MM')-1
  filepart2=fileimmr((ind1+5):ind2)
  filepart3=fileimmr((ind2+3):len(fileimmr))
  write(yyyystr,'(i4)') yyyy
  write(mmstr,'(i2.2)') mm
  filename=trim(filepart1)//yyyystr//trim(filepart2)//mmstr//trim(filepart3)

  ! open netcdf file
  write(logid,*) 'Reading the file: '//trim(filename)
  ierr=nf_open(trim(filename),NF_NOWRITE,nid)
  if(ierr.gt.0) then
    write(logid,*) 'Error opening file: '//trim(filename)
  endif

  ! if could not read file from previous month 
  if((jd-trajlen).lt.juldatei) then

    cnt=1

    ! inquire about variables
    ierr=nf_inq_dimid(nid,'x',xid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: x'
    endif
    ierr=nf_inq_dimid(nid,'y',yid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: y'
    endif
    ierr=nf_inq_dimid(nid,'presnivs',zid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: presnivs'
    endif
    ierr=nf_inq_dimid(nid,'time_counter',tid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: time_counter'
    endif

    ! read dimension sizes
    ierr=nf_inq_dimlen(nid,xid,nxini)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif
    ierr=nf_inq_dimlen(nid,yid,nyini)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif
    ierr=nf_inq_dimlen(nid,zid,nzini)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif
    ierr=nf_inq_dimlen(nid,tid,nt)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif

    ! allocate variables
    allocate(lonini(nxini))
    allocate(latini(nyini))
    allocate(lato(nyini))
    allocate(indy(nyini))
    allocate(pres(nzini))
    allocate(tmptime(nt))
    allocate(temp(nzini))
    allocate(tmp(nxini,nyini))
    allocate(tmpt(nxini,nyini,nzini,nt))
    allocate(tmpimmr(nxini,nyini,nzini,nt))

    ! allocate immrraw and timini with max time dimension
    allocate(immrraw(nxini,nyini,nzini,30))
    allocate(timini(30))

    ! read variables
    ierr=nf_inq_varid(nid,'nav_lon',lonid)
    ierr=nf_get_var_real(nid,lonid,tmp)
    lonini=tmp(:,1)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: nav_lon'
    endif
    ierr=nf_inq_varid(nid,'nav_lat',latid)
    ierr=nf_get_var_real(nid,latid,tmp)
    latini=tmp(1,:)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: nav_lat'
    endif
    ierr=nf_inq_varid(nid,'presnivs',prsid)
    ierr=nf_get_var_real(nid,prsid,pres)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: presnivs'
    endif
    ierr=nf_inq_varid(nid,'time_counter',timid)
    ierr=nf_get_var_real(nid,timid,tmptime)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: time_counter'
    endif
    ierr=nf_inq_varid(nid,trim(bginame),varid)
    ierr=nf_get_var_real(nid,varid,tmpimmr)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: ',trim(bginame)
    endif
    ierr=nf_inq_varid(nid,'temp',varid)
    ierr=nf_get_var_real(nid,varid,tmpt)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: temp'
    endif

  else

    ! if read previous month still need to read variables with time dimension

    ! inquire about dimension
    ierr=nf_inq_dimid(nid,'time_counter',tid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: time_counter'
    endif
    ! read dimension length
    ierr=nf_inq_dimlen(nid,tid,nt)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif

    ! allocate variables with time dimension
    allocate(tmptime(nt))
    allocate(tmpt(nxini,nyini,nzini,nt))
    allocate(tmpimmr(nxini,nyini,nzini,nt))

    ! read variables with time dimension 
    ierr=nf_inq_varid(nid,'time_counter',timid)
    ierr=nf_get_var_real(nid,timid,tmptime)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: time_counter'
    endif
    ierr=nf_inq_varid(nid,trim(bginame),varid)
    ierr=nf_get_var_real(nid,varid,tmpimmr)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: ',trim(bginame)
    endif
    ierr=nf_inq_varid(nid,'temp',varid)
    ierr=nf_get_var_real(nid,varid,tmpt)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: temp'
    endif

  endif

  ! close current month file
  ierr=nf_close(nid)

  ! assign current month to immrraw
  immrraw(:,:,:,cnt:(cnt+nt-1))=tmpimmr
  timini(cnt:(cnt+nt-1))=tmptime

  ! convert time into decimal year
  timini=timini/float(3600*24*365)+1979
 
  ! calculate average temperature at each pressure layer
  do i=1,nzini
    temp(i)=sum(tmpt(:,:,i,:))/(nxini*nyini*ntini)
  enddo

  ! if latitude order N-S convert to S-N
  lato=latini
  call sort(nyini,lato)
  ! indices to lat
  do iy=1,nyini
    indy(iy)=minloc(lato,dim=1,mask=lato.eq.latini(iy))
  end do
  immrraw=immrraw(:,indy,:,:)
  latini=lato

  deallocate(lato)
  deallocate(indy)
  deallocate(tmp)
  deallocate(tmpt)
  deallocate(tmptime)
  deallocate(tmpimmr)

  ntini=cnt+nt-1

end subroutine readlmdz

