!---------------------------------------------------------------------------------------
! FLEXINVERT : readtm5 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads mixing ratios from global fields from LMDZ model runs (LMDZ96 grid) 
!! note:
!!  - particular set-up is for LMDZ96 output files (runch4_online_test5)
!!  - time in LMDZ96 file is seconds since 1979-01-01 00:00:00
!!  - pressure in LMDZ96 is Pa
!!  - variable in LMDZ96 is volume mixing ratio
!---------------------------------------------------------------------------------------

subroutine readtm5(jd)

  use mod_var
  use mod_dates

  implicit none

  include 'netcdf.inc'

  character(len=100) :: filename,filepart1,filepart2,filepart3,filepart4
  character(len=2) :: ddstr,mmstr
  character(len=4) :: yyyystr
  integer :: iy,i,nti,it,iz
  integer :: ind1,ind2,ind3
  integer :: yyyymmddhh,yyyymmdd,yyyymm,yyyy,mm,dd,nday
  integer :: nid,yid,xid,zid,ierr,lonid,latid,prsid,tmid,timid,varid
  real :: eomday
  real(kind=8) jd,jdi,jdf,jdt
  real, dimension(26) :: plev
  real(kind=8), dimension(:), allocatable :: tmpti
  real, dimension(:,:), allocatable :: tmp
  real, dimension(:,:,:), allocatable :: var,tmpt

  ! TM5 pressure levels
  real, dimension(26) :: at = (/&
    0.000000,7.367743,210.393890,855.361755,&
    2063.779785,3850.913330,6144.314941,8802.356445,&
    11632.758789,14411.124023,16899.468750,18864.750000,&
    20097.402344,20429.863281,19755.109375,18045.183594,&
    15379.805664,12077.446289,8765.053711,6018.019531,&
    3960.291504,1680.640259,713.218079,298.495789,&
    95.636963,0.000000/)
  real, dimension(26) :: bt = (/&
    1.000000,0.994019,0.979663,0.951822,&
    0.907884,0.847375,0.771597,0.683269,&
    0.586168,0.484772,0.383892,0.288323,&
    0.202476,0.130023,0.073534,0.034121,&
    0.011143,0.001815,0.000076,0.000000,&
    0.000000,0.000000,0.000000,0.000000,&
    0.000000,0.000000/)

  ! TM5 latitude and longitude
  ! glb600x400
  allocate(lonini(60))
  lonini=(/&
   -180,-174,-168,-162,-156,-150,-144,-138,-132,-126,&
    -120,-114,-108,-102,-96,-90,-84,-78,-72,-66,& 
    -60,-54,-48,-42,-36,-30,-24,-18,-12,-6,&
    0,6,12,18,24,30,36,42,48,54,60,&
    66,72,78,84,90,96,102,108,114,120,&  
    126,132,138,144,150,156,162,168,174/)
  allocate(latini(45))
  latini=(/&
    -90,-86,-82,-78,-74,-70,-66,-62,-58,-54,&
    -50,-46,-42,-38,-34,-30,-26,-22,-18,-14,&
    -10,-6,-2,2,6,10,14,18,22,26,&
    30,34,38,42,46,50,54,58,62,66,&
    70,74,78,82,86/)

  ! calculate end date 
  yyyymmdd=caldate(jd)
  yyyymm=yyyymmdd/100
  eomday=calceomday(yyyymm)
  jdf=jd+eomday-1.

  ! calculate start date
  if((jd-trajlen).ge.juldatei) then 
    jdi=jd-trajlen
  else
    jdi=jd
  endif
  jdf=jd+eomday-1.
  nday=int(jdf-jdi)+1

  ! initialize 
  jdt=jdi
  i=0

  ! loop over daily files

  do while(jdt.le.jdf)

    i=i+1

    ! define exact filename
    yyyymmddhh=caldate(jdt)
    yyyymmdd=yyyymmddhh/100
    yyyy=yyyymmddhh/1000000
    mm=(yyyymmddhh-yyyy*1000000)/10000
    dd=(yyyymmddhh-yyyy*1000000-mm*10000)/100
    ind1=index(fileimmr,'YYYY')-1
    filepart1=fileimmr(1:ind1)
    ind2=index(fileimmr,'MM')-1
    filepart2=fileimmr((ind1+5):ind2)
    ind3=index(fileimmr,'DD')-1
    filepart3=fileimmr((ind2+3):ind3)
    filepart4=fileimmr((ind3+3):len(fileimmr))
    write(yyyystr,'(i4)') yyyy
    write(mmstr,'(i2.2)') mm
    write(ddstr,'(i2.2)') dd
    filename=trim(filepart1)//yyyystr//trim(filepart2)//mmstr//trim(filepart3)//ddstr//trim(filepart4)

    ! open netcdf file
    write(logid,*) 'Reading the file: '//trim(filename)
    ierr=nf_open(trim(filename),NF_NOWRITE,nid)
    if(ierr.gt.0) then
      write(logid,*) 'Error opening file: '//trim(filename)
    endif

    ! inquire about variables
    ierr=nf_inq_dimid(nid,'x',xid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: x'
    endif
    ierr=nf_inq_dimid(nid,'y',yid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: y'
    endif
    ierr=nf_inq_dimid(nid,'z',zid)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension: z'
    endif

    ! read dimension sizes
    ierr=nf_inq_dimlen(nid,xid,nxini)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif
    ierr=nf_inq_dimlen(nid,yid,nyini)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif
    ierr=nf_inq_dimlen(nid,zid,nzini)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading dimension length'
    endif

    ! allocate variables on first iteration
    if(jdt.eq.jdi) then
      allocate(pres(nzini))
      allocate(temp(nzini))
      allocate(timini(nday))
      allocate(tmpti(nday))
      allocate(tmp(nxini,nyini))
      allocate(var(nxini,nyini,nzini))
      allocate(tmpt(nxini,nyini,nzini))
      allocate(immrraw(nxini,nyini,nzini,nday))
    endif

    ! read variables
    ierr=nf_inq_varid(nid,'presm',prsid)
    ierr=nf_get_var_real(nid,prsid,tmp)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: presm'
    endif
    ierr=nf_inq_varid(nid,'tempm',tmid)
    ierr=nf_get_var_real(nid,tmid,tmpt)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: tempm'
    endif
    ierr=nf_inq_varid(nid,trim(bginame),varid)
    ierr=nf_get_var_real(nid,varid,var)
    if(ierr.gt.0) then
      write(logid,*) 'Error reading variable: ',trim(bginame)
    endif

    ierr=nf_close(nid)

    tmpti(i)=jdt
    ! assign mixing ratios and divide by 1e9
    immrraw(:,:,:,i)=var/1.e9

    jdt=jdt+1.

  end do ! jdt

  ! calculate average pressure at each level
  do i=1,nzini+1
    plev(i)=at(i)+bt(i)*sum(tmp)/float(nxini*nyini)
  enddo
  do i=1,nzini
    pres(i)=0.5*(plev(i)+plev(i+1))
    temp(i)=sum(tmpt(:,:,i))/float(nxini*nyini)
  enddo

  ! convert time into decimal year
  do i=1,nday
    yyyymmddhh=caldate(tmpti(i))
    yyyy=yyyymmddhh/1000000
    mm=(yyyymmddhh-1000000*yyyy)/10000
    dd=(yyyymmddhh-1000000*yyyy-10000*mm)/100
    timini(i)=float(yyyy)+float(mm-1)/12.+float(dd-1)/365.
  end do

  ntini=nday

  deallocate(tmp)
  deallocate(tmpti)
  deallocate(tmpt)
  deallocate(var)

end subroutine readtm5
