!---------------------------------------------------------------------------------------
! FLEXINVERT : regridoptimize
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> The operator from coarse to fine grid is an ambiguous, therefore find the optimal
!! fluxes at fine resolution based on the a priori distribution 
!!
!! x* = xb + BP^T(PBP^T+A)^-1(xa - Pxb)
!!
!! where:
!!   x* = posterior flux on fine grid (n)
!!   xb = prior flux on fine grid (n)
!!   xa = posterior flux on coarse grid (w)
!!   P  = operator from fine to coarse grid (w,n)
!!   B  = prior error covariance matrix on fine grid (n,n)
!!   A  = posterior error covariance on coarse grid (w,w)
!!   
!! Ref: Wu et al., J.Geophys.Res., 2011
!!
!! also find the new posteriori error covariance matrix on the fine grid, A*
!!
!! A* = (B^-1 + P^TA^-1P)^-1
!---------------------------------------------------------------------------------------

subroutine regridoptimize(nn,nw,covbfin,covbfinaw,covb,cova,xb,xa)

  use mod_var

  implicit none

  integer :: nn,nw,ierr
  integer :: M,N,lda,ldb,i,j,k,info
  real, dimension(nn*nemsteps) :: xb_fin,xa_fin
  real, dimension(nw*nemsteps+nbkgnd) :: xb,xa
  real, dimension(nw) :: xerr
  real, dimension(nn) :: erra_fin,errb_fin
  real, dimension(nw*nemsteps+nbkgnd,nw*nemsteps+nbkgnd) :: cova
  real(kind=8), dimension(nw*nemsteps+nbkgnd,nw*nemsteps+nbkgnd) :: covb
  real(kind=8), dimension(nn,nn) :: covbfin,covbfinaw
  integer :: ldu,ldvt,lwork
  integer, dimension(:), allocatable :: iwork
  real, dimension(nw,nn) :: prloper
  real(kind=8), dimension(:), allocatable :: work
  real(kind=8), dimension(:,:), allocatable :: U,UT,V,VT,iS,ViS,A,B
  real(kind=8), dimension(:), allocatable :: sigma
  real(kind=8), dimension(:,:), allocatable :: btp,btprl,pbtp,ipbtpa,btpipbtpa,icova,icovap
  real(kind=8), dimension(:,:), allocatable :: tpicovap,icovb_fin,icova_fin,cova_fin

  !--------------------------------------------------------------
  ! calculate state vector on fine grid 
  !--------------------------------------------------------------

  ! xb at fine resolution (no area weighting)
  do k=1,nemsteps
    do j=1,numynest
      do i=1,numxnest
        xb_fin((k-1)*nn+(j-1)*numxnest+i)=emisprior(ixl+i-1,iyl+j-1,k)
      end do
    end do
  end do

  ! calculate the prolongation operator
  ! row vectors have unit length 
  prloper(:,:)=0.
  do i=1,nw
    prloper(i,:)=real(gridnest(i,:),kind=4)/sqrt(real(sum(gridnest(i,:)),kind=4))
  end do

  ! calculate BP^T (no area weighting)
  allocate(btprl(nn,nw))
  btprl=matmul(covbfin,real(transpose(prloper),kind=8))

  ! calculate BP^T (with area weighting)
  allocate(btp(nn,nw))
  btp=matmul(covbfinaw,real(transpose(gridnest),kind=8))

  ! calculate PBP^T (with area weighting)
  allocate(pbtp(nw,nw))
  pbtp=matmul(real(gridnest,kind=8),btp)

  ! calculate (PBP^T+A)^(-1)
  ! use SVD (Cholesky fails for non-positive-definite matrices)
  allocate(A(nw,nw))
  M=nw                         ! number of rows of A
  N=nw                         ! number of columns of A
  A=pbtp+dble(cova(1:nw,1:nw)) ! matrix of dimension M x N
  lda=M                        ! leading dimension of A
  ldu=M                        ! leading dimension of U
  ldvt=N                       ! leading dimension of VT
  lwork=3*min(M,N)+max(max(M,N),4*min(M,N)*min(M,N)+4*min(M,N))
  allocate(work(lwork))
  allocate(iwork(8*min(M,N)))
  allocate(U(M,M))
  allocate(VT(N,N))
  allocate(sigma(N))
  call dgesdd('A',M,N,A,lda,sigma,U,ldu,VT,ldvt,work,lwork,iwork,info)
  if(info.ne.0) then
    print*, 'Error in singular value decomposition'
    if(info.lt.0) print*, 'the argument in position ',info,' had an illegal value'
    if(info.gt.0) print*, 'the algorithm failed to converge'
    stop
  endif
  deallocate(A)
  ! calculate (PBP^T+A)^(-1)
  allocate(UT(M,M))
  allocate(V(N,N))
  UT=transpose(U)
  V=transpose(VT)
  deallocate(U)
  deallocate(VT)
  allocate(iS(N,N))
  iS(:,:)=0.
  do i=1,nw
    if(sigma(i).ne.0.) iS(i,i)=1./sigma(i)
  end do
  allocate(ViS(N,N))
  allocate(ipbtpa(N,N))
  ViS=matmul(V,iS)
  ipbtpa=matmul(ViS,UT)
  deallocate(UT)
  deallocate(V)
  deallocate(sigma)
  deallocate(iS)
  deallocate(ViS)
  deallocate(iwork)
  deallocate(work)

  ! calculate BP^T(PBP^T+A)^(-1)
  allocate(btpipbtpa(nn,nw))
  btpipbtpa=matmul(btprl,ipbtpa)
  deallocate(btp)
  deallocate(btprl)
  deallocate(ipbtpa)

  ! calculate condition number of (PBP^T+A)
  call calcconditnum(nw,pbtp+dble(cova(1:nw,1:nw)))  
  deallocate(pbtp)

  ! calculate x* = xb_fin + BP^T(PBP^T+A)^-1(xa - xb) for each timestep
  do k=1,nemsteps
    ! calculate (xa - xb)
    xerr=xa((k-1)*nw+1:k*nw)-xb((k-1)*nw+1:k*nw)
    xa_fin((k-1)*nn+1:k*nn)=xb_fin((k-1)*nn+1:k*nn)+matmul(real(btpipbtpa,kind=4),xerr) 
  end do
  deallocate(btpipbtpa)

  !--------------------------------------------------------------
  ! calculate posterior error covariance matrix on fine grid
  !--------------------------------------------------------------

  ! calculate A^-1
  ! use SVD (Cholesky fails for non-positive-definite matrices)
  allocate(A(nw,nw))
  M=nw                      ! number of rows of A
  N=nw                      ! number of columns of A
  A=dble(cova(1:nw,1:nw))   ! matrix of dimension M x N
  ldb=M                     ! leading dimension of A
  ldu=M                     ! leading dimension of U
  ldvt=N                    ! leading dimension of VT
  lwork=3*min(M,N)+max(max(M,N),4*min(M,N)*min(M,N)+4*min(M,N))
  allocate(work(lwork))
  allocate(iwork(8*min(M,N)))
  allocate(U(M,M))
  allocate(VT(N,N))
  allocate(sigma(N))
  call dgesdd('A',M,N,A,ldb,sigma,U,ldu,VT,ldvt,work,-1,iwork,info)
  lwork=work(1)
  deallocate(work)
  allocate(work(lwork))
  call dgesdd('A',M,N,A,ldb,sigma,U,ldu,VT,ldvt,work,lwork,iwork,info)
  if(info.ne.0) then
    print*, 'Error in singular value decomposition'
    if(info.lt.0) print*, 'the argument in position ',info,' had an illegal value'
    if(info.gt.0) print*, 'the algorithm failed to converge'
    stop
  endif
  deallocate(A)
  ! calculate A^(-1)
  allocate(UT(M,M))
  allocate(V(N,N))
  UT=transpose(U)
  V=transpose(VT)
  deallocate(U)
  deallocate(VT)
  allocate(iS(N,N))
  iS(:,:)=0.
  do i=1,nw
    if(sigma(i).ne.0.) iS(i,i)=1./sigma(i)
  end do
  allocate(ViS(N,N))
  allocate(icova(N,N))
  ViS=matmul(V,iS)
  icova=matmul(ViS,UT)
  deallocate(UT)
  deallocate(V)
  deallocate(sigma)
  deallocate(iS)
  deallocate(ViS)
  deallocate(iwork)
  deallocate(work)

  ! calculate P^TA^-1P (use prolongation operator)
  allocate(icovap(nw,nn)) 
  icovap=matmul(icova,real(prloper,kind=8))
  deallocate(icova)
  allocate(tpicovap(nn,nn))
  tpicovap=matmul(real(transpose(prloper),kind=8),icovap)
  deallocate(icovap)

  ! calculate B^-1 (not area weighted)
  ! use SVD (Cholesky fails for non-positive-definite matrices)
  allocate(B(nn,nn))
  M=nn           ! number of rows of B
  N=nn           ! number of columns of B
  B=covbfin      ! matrix of dimension M x N
  ldb=M          ! leading dimension of B
  ldu=M          ! leading dimension of U
  ldvt=N         ! leading dimension of VT
  lwork=3*min(M,N)+max(max(M,N),4*min(M,N)*min(M,N)+4*min(M,N))
  allocate(work(lwork))
  allocate(iwork(8*min(M,N)))
  allocate(U(M,M))
  allocate(VT(N,N))
  allocate(sigma(N))
  call dgesdd('A',M,N,B,ldb,sigma,U,ldu,VT,ldvt,work,-1,iwork,info)
  lwork=work(1)
  deallocate(work)
  allocate(work(lwork))
  call dgesdd('A',M,N,B,ldb,sigma,U,ldu,VT,ldvt,work,lwork,iwork,info)
  if(info.ne.0) then
    print*, 'Error in singular value decomposition'
    if(info.lt.0) print*, 'the argument in position ',info,' had an illegal value'
    if(info.gt.0) print*, 'the algorithm failed to converge'
    stop
  endif
  deallocate(B)
  ! calculate B^(-1)
  allocate(UT(M,M))
  allocate(V(N,N))
  UT=transpose(U)
  V=transpose(VT)
  deallocate(U)
  deallocate(VT)
  allocate(iS(N,N))
  iS(:,:)=0.
  do i=1,nn
    if(sigma(i).ne.0.) iS(i,i)=1./sigma(i)
  end do
  allocate(ViS(N,N))
  allocate(icovb_fin(N,N))
  ViS=matmul(V,iS)
  icovb_fin=matmul(ViS,UT)
  deallocate(UT)
  deallocate(V)
  deallocate(sigma)
  deallocate(iS)
  deallocate(ViS)
  deallocate(iwork)
  deallocate(work)

  ! calculate (B^-1 + P^TA^-1P)
  allocate(icova_fin(nn,nn))
  icova_fin=icovb_fin+tpicovap
  deallocate(icovb_fin)
  deallocate(tpicovap)

  ! calculate A = (B^-1 + P^TA^-1P)^-1
  ! use SVD (Cholesky fails for non-positive-definite matrices)
  allocate(A(nn,nn))
  M=nn               ! number of rows of A
  N=nn               ! number of columns of A
  A=icova_fin        ! matrix of dimension M x N
  lda=M              ! leading dimension of A
  ldu=M              ! leading dimension of U
  ldvt=N             ! leading dimension of VT
  lwork=3*min(M,N)+max(max(M,N),4*min(M,N)*min(M,N)+4*min(M,N))
  allocate(work(lwork))
  allocate(iwork(8*min(M,N)))
  allocate(U(M,M))
  allocate(VT(N,N))
  allocate(sigma(N))
  call dgesdd('A',M,N,A,lda,sigma,U,ldu,VT,ldvt,work,-1,iwork,info)
  lwork=work(1)
  deallocate(work)
  allocate(work(lwork))
  call dgesdd('A',M,N,A,lda,sigma,U,ldu,VT,ldvt,work,lwork,iwork,info)
  if(info.ne.0) then
    print*, 'Error in singular value decomposition'
    if(info.lt.0) print*, 'the argument in position ',info,' had an illegal value'
    if(info.gt.0) print*, 'the algorithm failed to converge'
    stop
  endif
  deallocate(A)
  ! calculate A
  allocate(UT(M,M))
  allocate(V(N,N))
  UT=transpose(U)
  V=transpose(VT)
  deallocate(U)
  deallocate(VT)
  allocate(iS(N,N))
  iS(:,:)=0.
  do i=1,nn
    if(sigma(i).ne.0.) iS(i,i)=1./sigma(i)
  end do
  allocate(ViS(N,N))
  allocate(cova_fin(N,N))
  ViS=matmul(V,iS)
  cova_fin=matmul(ViS,UT)
  deallocate(UT)
  deallocate(V)
  deallocate(sigma)
  deallocate(iS)
  deallocate(ViS)
  deallocate(iwork)
  deallocate(work)
  deallocate(icova_fin)

  ! calculate the posterior error
!  do i=1,nn
!    erra_fin(i)=real(sqrt(cova_fin(i,i)),kind=4)
!  end do
!  deallocate(cova_fin)

  ! calculate the prior and posterior errors with covariances
  do i=1,nn
    errb_fin(i)=real(sqrt(sum(abs(covbfin(:,i)))),kind=4)
    erra_fin(i)=real(sqrt(sum(abs(cova_fin(:,i)))),kind=4)
  end do
  deallocate(cova_fin)

  ! regrid to 3D
  do k=1,nemsteps
    do j=1,numynest
      do i=1,numxnest  
        emispost(i,j,k)=xa_fin((k-1)*nn+(j-1)*numxnest+i)
        errpost(i,j,k)=erra_fin((j-1)*numxnest+i)
        errpri(i,j,k)=errb_fin((j-1)*numxnest+i)
      end do
    end do
  end do


end subroutine regridoptimize

